package com.sd.business.controller.order;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.fjhx.flow.enums.FlowStatusEnum;
import com.ruoyi.common.core.domain.BaseSelectDto;
import com.sd.business.entity.order.dto.OrderInfoDto;
import com.sd.business.entity.order.dto.OrderSelectDto;
import com.sd.business.entity.order.dto.SkuSpecPriceDto;
import com.sd.business.entity.order.enums.OrderExceptionTypeEnum;
import com.sd.business.entity.order.vo.CompareVo;
import com.sd.business.entity.order.vo.OrderInfoVo;
import com.sd.business.entity.order.vo.SkuSpecPriceVo;
import com.sd.business.entity.production.vo.OutBomVo;
import com.sd.business.service.order.OrderService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;
import java.util.Map;

/**
 * <p>
 * 订单 前端控制器
 * </p>
 *
 * @author
 * @since 2023-06-30
 */
@RestController
@RequestMapping("/orderInfo")
public class OrderController {

    @Autowired
    private OrderService orderService;

    /**
     * 订单分页
     */
    @PostMapping("/page")
    public Page<OrderInfoVo> page(@RequestBody OrderSelectDto dto) {
        return orderService.getPage(dto);
    }

    /**
     * 订单明细
     */
    @PostMapping("/detail")
    public OrderInfoVo detail(@RequestBody BaseSelectDto dto) {
        return orderService.detail(dto.getId());
    }

    /**
     * 订单新增
     */
    @PostMapping("/add")
    public void add(@Validated @RequestBody OrderInfoDto orderDto) {
        orderDto.setFlowStatus(FlowStatusEnum.PASS.getKey());
        orderService.add(orderDto);
    }

    /**
     * 订单编辑
     */
    @PostMapping("/edit")
    public void edit(@RequestBody OrderInfoDto orderDto) {
        orderService.edit(orderDto);
    }

    /**
     * 订单确认
     */
    @PostMapping("/confirmation")
    public void confirmation(@RequestBody OrderInfoDto orderDto) {
        orderService.confirmation(orderDto);
    }

    /**
     * 订单删除
     */
    @PostMapping("/delete")
    public void delete(@RequestBody BaseSelectDto dto) {
        orderService.delete(dto.getId(), true);
    }

    /**
     * 通过sku规格id获取产品价格
     */
    @PostMapping("/getSkuSpecPrice")
    public SkuSpecPriceVo getSkuSpecPrice(@RequestBody SkuSpecPriceDto dto) {
        return orderService.getSkuSpecPrice(dto);
    }

    /**
     * 通过sku规格id获取产品价格
     */
    @PostMapping("/deletedOrderPage")
    public Page<OrderInfoVo> deletedOrderPage(@RequestBody OrderSelectDto dto) {
        return orderService.deletedOrderPage(dto);
    }

    /**
     * 通过sku规格id获取产品价格
     */
    @PostMapping("/deletedOrderDetail")
    public OrderInfoVo deletedOrderDetail(@RequestBody BaseSelectDto dto) {
        return orderService.deletedOrderDetail(dto);
    }

    /**
     * 转为正常订单
     */
    @PostMapping("/turnToRegularOrder")
    public void turnToRegularOrder(@RequestBody BaseSelectDto dto) {
        orderService.turnToRegularOrder(dto.getId());
    }

    /**
     * 订单删除并回滚库存
     */
    @PostMapping("/deleteAndStore")
    public void deleteAndStore(@RequestBody BaseSelectDto dto) {
        orderService.deleteAndStore(dto.getId());
    }

    /**
     * 订单对比
     */
    @PostMapping("/compare")
    public CompareVo compare(@RequestBody OrderSelectDto dto) {
        return orderService.compare(dto);
    }

    /**
     * 获取订单异常类型
     */
    @PostMapping("/getExceptionTypeList")
    public List<Map<String, Object>> getExceptionTypeList() {
        return OrderExceptionTypeEnum.getMapList();
    }

    /**
     * 获取万里牛单号
     */
    @PostMapping("/getOrderWlnCodeStr")
    public String getOrderWlnCodeStr(@RequestBody OrderSelectDto dto) {
        return orderService.getOrderWlnCodeStr(dto);
    }


    /**
     * 获取订单送货单号
     */
    @PostMapping("/getOrderDeliveryCode")
    public String getOrderDeliveryCode() {
        return orderService.getOrderDeliveryCode();
    }

    /**
     * 保存订单送货单号
     * @param orderDto
     */
    @PostMapping("/saveOrderDeliveryCode")
    public void saveOrderDeliveryCode(@RequestBody OrderInfoDto orderDto) {
        orderService.saveOrderDeliveryCode(orderDto);
    }

    /**
     * 挂起订单
     */
    @PostMapping("/suspendOrder")
    public void suspendOrder(@RequestBody BaseSelectDto dto) {
        orderService.suspendOrder(dto.getId());
    }


    /**
     * 取消挂起订单
     */
    @PostMapping("/cancelSuspendOrder")
    public void cancelSuspendOrder(@RequestBody BaseSelectDto dto) {
        orderService.cancelSuspendOrder(dto.getId());
    }

    /**
     * 订单删除退料清单
     */
    @PostMapping("/returnBomList")
    public List<OutBomVo> returnBomList(@RequestBody BaseSelectDto dto) {
        return orderService.returnBomList(dto.getId());
    }

}
