package com.sd.business.controller.production;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.sd.business.entity.production.dto.StockPreparationDto;
import com.sd.business.entity.production.vo.*;
import com.sd.business.service.production.StockPreparationService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import javax.validation.constraints.NotEmpty;
import java.util.List;

/**
 * 生产备料
 */
@RestController
@RequestMapping("/stockPreparation")
public class StockPreparationController {

    @Autowired
    private StockPreparationService stockPreparationService;

    /**
     * 待备料列表
     */
    @PostMapping("/uncompletedList")
    public List<UncompletedVo> uncompletedList(@RequestBody StockPreparationDto dto) {
        return stockPreparationService.uncompletedList(dto);
    }

    /**
     * 打印待备料列表
     */
    @PostMapping("/printUncompletedList")
    public List<PrintUncompletedVo> printUncompletedList(@RequestBody StockPreparationDto dto) {
        return stockPreparationService.printUncompletedList(dto);
    }

    /**
     * 已备料分页
     */
    @PostMapping("/completedPage")
    public Page<StockPreparationVo> completedPage(@RequestBody StockPreparationDto dto) {
        return stockPreparationService.completedPage(dto);
    }

    /**
     * 提交备料
     */
    @PostMapping("/submit")
    public void submit(@RequestBody StockPreparationDto dto) {
        stockPreparationService.submit(dto);
    }

    /**
     * 获取包材
     */
    @PostMapping("/getPackageBomList")
    public List<PackageBomVo> getPackageBomList(@Valid @NotEmpty(message = "订单skuId不能为空") @RequestBody List<Long> orderSkuIdList) {
        return stockPreparationService.getPackageBomList(orderSkuIdList);
    }

    /**
     * 待出库列表
     */
    @PostMapping("/outBomList")
    public List<OutBomVo> outBomList(@RequestBody StockPreparationDto dto) {
        return stockPreparationService.outBomList(dto);
    }

}
