package com.sd.business.controller.sku;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ruoyi.common.core.domain.BaseSelectDto;
import com.sd.business.entity.sku.dto.*;
import com.sd.business.entity.sku.po.Sku;
import com.sd.business.entity.sku.vo.ReplacePackagingMaterialVo;
import com.sd.business.entity.sku.vo.ReplacePrincipalMaterialVo;
import com.sd.business.entity.sku.vo.SkuVo;
import com.sd.business.service.sku.SkuService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;


/**
 * <p>
 * sku 前端控制器
 * </p>
 *
 * @author
 * @since 2023-07-03
 */
@RestController
@RequestMapping("/sku")
public class SkuController {

    @Autowired
    private SkuService skuService;

    /**
     * sku列表
     */
    @PostMapping("/list")
    public List<Sku> list() {
        return skuService.list();
    }

    /**
     * sku分页
     */
    @PostMapping("/page")
    public Page<SkuVo> page(@RequestBody SkuSelectDto dto) {
        return skuService.getPage(dto);
    }

    /**
     * sku明细
     */
    @PostMapping("/detail")
    public SkuVo detail(@RequestBody BaseSelectDto dto) {
        return skuService.detail(dto.getId());
    }

    /**
     * sku新增
     */
    @PostMapping("/add")
    public void add(@Validated @RequestBody SkuDto skuDto) {
        skuService.add(skuDto);
    }

    /**
     * sku编辑
     */
    @PostMapping("/edit")
    public void edit(@RequestBody SkuDto skuDto) {
        skuService.edit(skuDto);
    }

    /**
     * sku删除
     */
    @PostMapping("/delete")
    public void delete(@RequestBody BaseSelectDto dto) {
        skuService.delete(dto.getId());
    }

    /**
     * 获取可替换主材列表
     */
    @PostMapping("/getReplacePrincipalMaterialPage")
    public Page<ReplacePrincipalMaterialVo> getReplacePrincipalMaterialPage(@RequestBody GetReplaceMaterialListDto dto) {
        return skuService.getReplacePrincipalMaterialPage(dto);
    }

    /**
     * 替换主材
     */
    @PostMapping("/replacePrincipalMaterial")
    public void replacePrincipalMaterial(@Validated @RequestBody ReplaceMaterialDto dto) {
        skuService.replacePrincipalMaterial(dto);
    }

    /**
     * 获取可替换包材列表
     */
    @PostMapping("/getReplacePackagingMaterialPage")
    public Page<ReplacePackagingMaterialVo> getReplacePackagingMaterialPage(@RequestBody GetReplaceMaterialListDto dto) {
        return skuService.getReplacePackagingMaterialPage(dto);
    }

    /**
     * 替换包材
     */
    @PostMapping("/replacePackagingMaterial")
    public void replacePackagingMaterial(@Validated @RequestBody ReplacePackagingMaterialDto dto) {
        skuService.replacePackagingMaterial(dto);
    }

    /**
     * 删除包材
     */
    @PostMapping("/removePackagingMaterial")
    public void removePackagingMaterial(@RequestBody ReplacePackagingMaterialDto dto) {
        skuService.removePackagingMaterial(dto);
    }


}
