package com.sd.business.service.apply.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.excel.EasyExcel;
import com.baomidou.dynamic.datasource.annotation.DSTransactional;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.file.entity.FileInfo;
import com.fjhx.file.entity.ObsFile;
import com.fjhx.file.service.FileInfoService;
import com.fjhx.file.utils.ObsFileUtil;
import com.ruoyi.common.core.domain.BaseIdPo;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.sd.business.entity.apply.dto.ApplyBuyDto;
import com.sd.business.entity.apply.dto.ApplyBuySelectDto;
import com.sd.business.entity.apply.po.ApplyBuy;
import com.sd.business.entity.apply.po.ApplyBuyBom;
import com.sd.business.entity.apply.vo.ApplyBuyBomAccessoryVo;
import com.sd.business.entity.apply.vo.ApplyBuyBomVo;
import com.sd.business.entity.apply.vo.ApplyBuyVo;
import com.sd.business.entity.bom.bo.BomSpecOrderConsumptionBo;
import com.sd.business.entity.bom.po.BomSpec;
import com.sd.business.entity.department.constant.DepartmentConstant;
import com.sd.business.entity.inventory.po.Inventory;
import com.sd.business.entity.purchase.vo.PurchaseInTransitBomVo;
import com.sd.business.entity.warehouse.constant.WarehouseConstant;
import com.sd.business.mapper.apply.ApplyBuyMapper;
import com.sd.business.service.apply.ApplyBuyBomService;
import com.sd.business.service.apply.ApplyBuyService;
import com.sd.business.service.bom.BomSpecService;
import com.sd.business.service.inventory.InventoryService;
import com.sd.business.service.order.OrderSalesShipmentStatisticsService;
import com.sd.business.service.order.OrderService;
import com.sd.business.service.purchase.PurchaseBomService;
import com.sd.business.upload.ObsUploadService;
import com.sd.business.util.CodeEnum;
import com.sd.framework.util.Assert;
import com.sd.framework.util.StreamUtil;
import com.sd.framework.util.excel.util.ExcelUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.stream.Collectors;


/**
 * <p>
 * 申购单 服务实现类
 * </p>
 *
 * @author
 * @since 2023-06-30
 */
@Service
public class ApplyBuyServiceImpl extends ServiceImpl<ApplyBuyMapper, ApplyBuy> implements ApplyBuyService {

    @Autowired
    private ApplyBuyBomService applyBuyBomService;

    @Autowired
    private BomSpecService bomSpecService;

    @Autowired
    private PurchaseBomService purchaseBomService;

    @Autowired
    private OrderSalesShipmentStatisticsService orderSalesShipmentStatisticsService;

    @Autowired
    private InventoryService inventoryService;

    @Autowired
    private FileInfoService fileInfoService;

    @Autowired
    private OrderService orderService;

    @Autowired
    private ObsUploadService obsUploadService;

    @Override
    public Page<ApplyBuyVo> getPage(ApplyBuySelectDto dto) {

        IWrapper<ApplyBuy> wrapper = getWrapper();
        wrapper.orderByDesc("ab", ApplyBuy::getId);

        // 申购单号
        wrapper.like("ab", ApplyBuy::getCode, dto.getCode());
        // 状态
        wrapper.eq("ab", ApplyBuy::getFlowStatus, dto.getFlowStatus());
        // 申购开始时间
        wrapper.ge("ab", ApplyBuy::getApplyTime, dto.getBeginTime());
        // 申购结束时间
        wrapper.le("ab", ApplyBuy::getApplyTime, dto.getEndTime());

        return this.baseMapper.getPage(dto.getPage(), wrapper);
    }

    @Override
    public ApplyBuyVo detail(Long id) {
        Assert.notNull(id, "申购单id不能为空");

        // 申购单
        ApplyBuy ApplyBuy = this.getById(id);
        Assert.notNull(ApplyBuy, "没有找到申购单");

        // 申购明细
        List<ApplyBuyBom> list = applyBuyBomService.list(q -> q.eq(ApplyBuyBom::getApplyBuyId, id));
        List<ApplyBuyBomVo> applyBuyBomList = BeanUtil.copyToList(list, ApplyBuyBomVo.class);

        // 赋值bom规格
        bomSpecService.attributeAssign(applyBuyBomList, ApplyBuyBom::getBomSpecId, (item, bomSpec) -> {
            item.setBomSpecCode(bomSpec.getCode());
            item.setBomSpecName(bomSpec.getName());
            item.setBomSpecColour(bomSpec.getColour());
            item.setBomSpecLength(bomSpec.getLength());
            item.setBomSpecWidth(bomSpec.getWidth());
            item.setBomSpecHeight(bomSpec.getHeight());
        });

        // TODO 库存，在途数量，近30天消耗量，近30天入库量，安全库存 未赋值

        // 赋值返回值
        ApplyBuyVo result = BeanUtil.toBean(ApplyBuy, ApplyBuyVo.class);
        result.setApplyBuyBomList(applyBuyBomList);
        return result;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void add(ApplyBuyDto dto) {
        Long applyBuyId = IdWorker.getId();

        dto.setId(applyBuyId);
        dto.setCode(CodeEnum.APPLY_BUY_CODE.getCode());

        List<ApplyBuyBom> applyBuyBomList = dto.getApplyBuyBomList();
        applyBuyBomList.forEach(item -> {
            item.setApplyBuyId(applyBuyId);
            item.setPurchaseQuantity(BigDecimal.ZERO);
            item.setFrozenQuantity(BigDecimal.ZERO);
        });

        save(dto);
        applyBuyBomService.saveBatch(applyBuyBomList);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void edit(ApplyBuyDto dto) {
        Long applyBuyId = dto.getId();

        List<ApplyBuyBom> applyBuyBomList = dto.getApplyBuyBomList();
        applyBuyBomList.forEach(item -> item.setApplyBuyId(applyBuyId));

        updateById(dto);
        applyBuyBomService.editLinked(applyBuyBomList, ApplyBuyBom::getApplyBuyId, applyBuyId);
    }

    @DSTransactional
    @Override
    public void addBomAccessory(Long id) {
        // 新增备货需求附件
        List<ApplyBuyBomAccessoryVo> applyData = this.getApplyData(id);
        // 修改表头
        Map<String, String> keys = new HashMap<>();
        Date date = DateUtil.date();
        Date lastMonthDate = DateUtil.offsetMonth(date, -1);
        Date beforeLastMonthDate = DateUtil.offsetMonth(date, -2);
        keys.put("currentMonthSales", (DateUtil.month(date) + 1) + "月销量");
        keys.put("lastMonthSales", (DateUtil.month(lastMonthDate) + 1) + "月销量");
        keys.put("beforeLastMonthSales", (DateUtil.month(beforeLastMonthDate) + 1) + "月销量");

        ByteArrayOutputStream os = new ByteArrayOutputStream();
        EasyExcel.write(os, ExcelUtil.getClassNew(new ApplyBuyBomAccessoryVo(), keys)).sheet("sheetName").doWrite(applyData);
        String fileName = "胜德体育备货需求.xlsx";
        // 上传文件
        ByteArrayInputStream bis = new ByteArrayInputStream(os.toByteArray());
        String fileUrl = obsUploadService.uploadFileByte(bis, fileName);

        // 保存文件
        FileInfo fileInfo = new FileInfo();
        fileInfo.setFileUrl(fileUrl);
        fileInfo.setFileName(fileName);
        fileInfoService.save(fileInfo);
        // 新增附件列表
        ObsFile obsFile = new ObsFile();
        obsFile.setId(fileInfo.getId());
        obsFile.setFileName(fileName);
        obsFile.setFileUrl(fileUrl);
        ObsFileUtil.saveFile(obsFile, id);
    }


    /**
     * 获取需求单数据
     * @param applyBuyId
     * @return
     */
    private List<ApplyBuyBomAccessoryVo> getApplyData(Long applyBuyId) {
        List<ApplyBuyBomAccessoryVo> accessoryList = new ArrayList<>();
        List<ApplyBuyBom> list = applyBuyBomService.list(q -> q.eq(ApplyBuyBom::getApplyBuyId, applyBuyId));
        List<Long> bomSpecIds = list.stream().map(ApplyBuyBom::getBomSpecId).collect(Collectors.toList());
        Map<Long, BomSpec> bomSpecMap = bomSpecService.mapKEntity(BaseIdPo::getId, q -> q.in(BaseIdPo::getId, bomSpecIds));
        Map<Long, BigDecimal> turnoverRateMap = orderSalesShipmentStatisticsService.getTurnoverRateMapByBomSpecIds(bomSpecIds);
        // 在途数据
        List<PurchaseInTransitBomVo> purchaseBomInTransitList = purchaseBomService.getPurchaseBomInTransitSum(bomSpecIds);
        Map<Long, BigDecimal> inTransitMap = purchaseBomInTransitList.stream().collect(Collectors.toMap(PurchaseInTransitBomVo::getBomSpecId, PurchaseInTransitBomVo::getInTransitQuantity));

        // 当月时间
        Date date = DateUtil.date();
        // 上上个月时间
        Date beforeLastMonthDate = DateUtil.offsetMonth(date, -2);
        Date beforeLastMonthBeginDate = DateUtil.beginOfMonth(beforeLastMonthDate);
        // 查询近3个月的bom消耗数据
        List<BomSpecOrderConsumptionBo> orderBomSpecOutStorageList = orderService.getOrderBomSpecOutStorageQuantity(bomSpecIds, beforeLastMonthBeginDate, date);
        Map<Long, List<BomSpecOrderConsumptionBo>> bomSpecOutStorageMap = orderBomSpecOutStorageList.stream().collect(Collectors.groupingBy(BomSpecOrderConsumptionBo::getBomSpecId));

        for (ApplyBuyBom applyBuyBom : list) {
            Long bomSpecId = applyBuyBom.getBomSpecId();
            BomSpec bomSpec = bomSpecMap.get(bomSpecId);
            ApplyBuyBomAccessoryVo vo = new ApplyBuyBomAccessoryVo();
            List<BomSpecOrderConsumptionBo> bomSpecConsumptionList = bomSpecOutStorageMap.getOrDefault(bomSpecId, Collections.emptyList());
            // 当月消耗量
            Date currentMonthBeginDate = DateUtil.beginOfMonth(date);
            BigDecimal currentMonthSalesQuantity = bomSpecConsumptionList.stream()
                    .filter(item -> currentMonthBeginDate.compareTo(item.getConsumptionDate()) <= 0 && date.compareTo(item.getConsumptionDate()) >= 0)
                    .map(BomSpecOrderConsumptionBo::getQuantity)
                    .reduce(BigDecimal.ZERO, BigDecimal::add);
            // 上个月消耗量
            Date lastMonthDate = DateUtil.offsetMonth(date, -1);
            Date lastMonthBeginDate = DateUtil.beginOfMonth(lastMonthDate);
            Date lastMonthEndDate = DateUtil.endOfMonth(lastMonthDate);
            BigDecimal lastMonthSalesQuantity = bomSpecConsumptionList.stream()
                    .filter(item -> lastMonthBeginDate.compareTo(item.getConsumptionDate()) <= 0 && lastMonthEndDate.compareTo(item.getConsumptionDate()) >= 0)
                    .map(BomSpecOrderConsumptionBo::getQuantity)
                    .reduce(BigDecimal.ZERO, BigDecimal::add);
            // 上上个月消耗量
            Date beforeLastMonthEndDate = DateUtil.endOfMonth(beforeLastMonthDate);
            BigDecimal beforeLastMonth00Quantity = bomSpecConsumptionList.stream()
                    .filter(item -> beforeLastMonthBeginDate.compareTo(item.getConsumptionDate()) <= 0 && beforeLastMonthEndDate.compareTo(item.getConsumptionDate()) >= 0)
                    .map(BomSpecOrderConsumptionBo::getQuantity)
                    .reduce(BigDecimal.ZERO, BigDecimal::add);
            List<Inventory> inventoryList = inventoryService.list(q -> q
                    .eq(Inventory::getBomSpecId, bomSpecId)
                    .eq(Inventory::getDepartmentId, DepartmentConstant.SD_SPORTS)
                    .in(Inventory::getWarehouseId, WarehouseConstant.SEMI_FINISHED_PRODUCT, WarehouseConstant.PACKAGING_MATERIAL));
            // 当前库存数
            BigDecimal inventoryQuantity = ObjectUtil.isEmpty(inventoryList) ? BigDecimal.ZERO : StreamUtil.bigDecimalAdd(inventoryList, Inventory::getQuantity);
            // 在途数量
            BigDecimal inTransitSum = inTransitMap.getOrDefault(bomSpecId, BigDecimal.ZERO);
            // 月平均消耗
            BigDecimal averageMonthSales = currentMonthSalesQuantity
                    .add(lastMonthSalesQuantity)
                    .add(beforeLastMonth00Quantity)
                    .divide(new BigDecimal(90), 2, RoundingMode.HALF_UP);
            // 实际销量可消耗天数
            BigDecimal actualSalesDays;
            if (ObjectUtil.equals(averageMonthSales, BigDecimal.ZERO)) {
                actualSalesDays = BigDecimal.ZERO;
            } else {
                actualSalesDays = inventoryQuantity.divide(averageMonthSales, 0, RoundingMode.HALF_UP);
            }
            // 短缺数量合计
            BigDecimal shortageQuantity;
            if (ObjectUtil.equals(currentMonthSalesQuantity, BigDecimal.ZERO)) {
                shortageQuantity = BigDecimal.ZERO;
            } else {
                shortageQuantity = inventoryQuantity
                        .add(inTransitSum)
                        .subtract(currentMonthSalesQuantity
                                .divide(new BigDecimal(90), 2, RoundingMode.HALF_UP)
                                .multiply(new BigDecimal(60))
                        );
            }
            // 规格
            String length = bomSpec.getLength() == null ? "0" : bomSpec.getLength().stripTrailingZeros().toPlainString();
            String width = bomSpec.getWidth() == null ? "0" : bomSpec.getWidth().stripTrailingZeros().toPlainString();
            String height = bomSpec.getHeight() == null ? "0" : bomSpec.getHeight().stripTrailingZeros().toPlainString();
            // 赋值
            vo.setBomSpecCode(bomSpec.getCode());
            vo.setBomSpecName(bomSpec.getName());
            vo.setSpecification(length + " * " + width + " * " + height);
            vo.setColour(bomSpec.getName());
            vo.setStockQuantity(inventoryQuantity);
            vo.setInTransitQuantity(inTransitSum);
            vo.setCurrentMonthSales(currentMonthSalesQuantity);
            vo.setLastMonthSales(lastMonthSalesQuantity);
            vo.setBeforeLastMonthSales(beforeLastMonth00Quantity);
            vo.setAverageMonthSales(averageMonthSales);
            vo.setActualSalesDays(actualSalesDays);
            vo.setShortageQuantity(shortageQuantity);
            vo.setApplyBuyQuantity(applyBuyBom.getQuantity());
            vo.setTurnoverRate(turnoverRateMap.get(applyBuyBom.getBomSpecId()));
            accessoryList.add(vo);
        }

        return accessoryList;
    }

}
