package com.sd.business.service.bom.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.constant.StatusConstant;
import com.ruoyi.common.core.domain.BaseIdPo;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.sd.business.entity.bom.dto.BomSpecDto;
import com.sd.business.entity.bom.dto.BomSpecSelectDto;
import com.sd.business.entity.bom.dto.BomSpecUpdateBoardDto;
import com.sd.business.entity.bom.dto.BomSpecUpdateBoardSortDto;
import com.sd.business.entity.bom.po.BomSpec;
import com.sd.business.entity.bom.vo.BomSpecBoardSelectVo;
import com.sd.business.entity.bom.vo.BomSpecVo;
import com.sd.business.entity.department.constant.DepartmentConstant;
import com.sd.business.entity.inventory.po.Inventory;
import com.sd.business.entity.warehouse.constant.WarehouseConstant;
import com.sd.business.mapper.bom.BomSpecMapper;
import com.sd.business.service.bom.BomSpecService;
import com.sd.business.service.inventory.InventoryService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;


/**
 * <p>
 * bom规格 服务实现类
 * </p>
 *
 * @author
 * @since 2023-06-30
 */
@Service
public class BomSpecServiceImpl extends ServiceImpl<BomSpecMapper, BomSpec> implements BomSpecService {
    @Autowired
    private InventoryService inventoryService;

    @Override
    public Page<BomSpecVo> getPage(BomSpecSelectDto dto) {
        IWrapper<BomSpec> wrapper = getWrapper();
        wrapper.like("bs", BomSpec::getName, dto.getName());
        wrapper.like("bs", BomSpec::getCode, dto.getCode());
        wrapper.orderByDesc("bs", BomSpec::getId);
        Page<BomSpecVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        return page;
    }

    @Override
    public BomSpecVo detail(Long id) {
        BomSpec bomSpec = this.getById(id);
        BomSpecVo result = BeanUtil.toBean(bomSpec, BomSpecVo.class);
        return result;
    }

    @Override
    public void add(BomSpecDto bomSpecDto) {
        this.save(bomSpecDto);
    }

    @Override
    public void edit(BomSpecDto bomSpecDto) {
        this.updateById(bomSpecDto);
    }

    @Override
    public void delete(Long id) {
        this.removeById(id);
    }

    @Override
    public void editBoard(BomSpecUpdateBoardDto dto) {
        List<Long> bomSpecIdList = dto.getBomSpecIdList();
        this.update(q -> q.set(BomSpec::getIsShowBoard, StatusConstant.NO).set(BomSpec::getSort, null).eq(BomSpec::getIsShowBoard, StatusConstant.YES));
        if (ObjectUtil.isNotEmpty(bomSpecIdList)) {
            for (int i = 0; i < bomSpecIdList.size(); i++) {
                Long bomSpecId = bomSpecIdList.get(i);
                int sort = i;
                this.update(q -> q.set(BomSpec::getIsShowBoard, StatusConstant.YES)
                        .set(BomSpec::getSort, sort).eq(BomSpec::getId, bomSpecId));
            }
        }
    }

    @Override
    public List<BomSpecBoardSelectVo> getBoardList() {
        List<BomSpec> list = this.list(q -> q.eq(BomSpec::getIsShowBoard, StatusConstant.YES).orderByAsc(BomSpec::getSort));
        List<Long> bomSpecIds = list.stream().map(BaseIdPo::getId).collect(Collectors.toList());
        // 查询半成品和包材库存
        List<Long> warehouseIds = new ArrayList<>();
        warehouseIds.add(WarehouseConstant.SEMI_FINISHED_PRODUCT);
        warehouseIds.add(WarehouseConstant.PACKAGING_MATERIAL);

        Map<Long, Inventory> inventoryMap = inventoryService.mapKEntity(Inventory::getBomSpecId, q -> q.eq(Inventory::getDepartmentId, DepartmentConstant.SD_SPORTS)
                .in(Inventory::getWarehouseId, warehouseIds)
                .in(Inventory::getBomSpecId, bomSpecIds));
        List<BomSpecBoardSelectVo> boardSelectList = BeanUtil.copyToList(list, BomSpecBoardSelectVo.class);
        List<BomSpecBoardSelectVo> collect = boardSelectList.stream().peek(item -> {
            Inventory inventory = inventoryMap.get(item.getId());
            if (inventory == null) {
                item.setInventoryQuantity(BigDecimal.ZERO);
            } else {
                BigDecimal inventoryQuantity = inventory.getQuantity().add(inventory.getLockQuantity() == null ? BigDecimal.ZERO : inventory.getLockQuantity());
                item.setInventoryQuantity(inventoryQuantity);
            }
        }).collect(Collectors.toList());
        return collect;
    }

    @Override
    public void editBoardSort(BomSpecUpdateBoardSortDto dto) {
        BomSpec bomSpec = this.getById(dto.getId());
        if (bomSpec != null && !Objects.equals(dto.getCurrentSort(), bomSpec.getSort())) {
            Integer oldSort = bomSpec.getSort();
            Integer currentSort = dto.getCurrentSort();
            // 如果当前排序大于原排序，当前排序到原排序之间 -1
            if (oldSort < currentSort) {
                this.update(q -> q.setSql("sort = sort - 1").le(BomSpec::getSort, currentSort).gt(BomSpec::getSort, oldSort));
            } else {
                this.update(q -> q.setSql("sort = sort + 1").lt(BomSpec::getSort, oldSort).ge(BomSpec::getSort, currentSort));
            }
            // 更新当前bom排序
            this.update(q -> q.set(BomSpec::getSort, currentSort).eq(BomSpec::getId, bomSpec.getId()));
        }
    }

    @Override
    public void editPrice(BomSpecDto bomSpecDto) {
        BomSpec bomSpec = this.getById(bomSpecDto.getId());
        if (bomSpec == null) {
            throw new ServiceException("未知bom规格id!");
        }
        bomSpec.setCostPrice(bomSpecDto.getCostPrice());
        bomSpec.setInternalSellingPrice(bomSpecDto.getInternalSellingPrice());
        bomSpec.setExternalSellingPrice(bomSpecDto.getExternalSellingPrice());
        this.updateById(bomSpec);
    }
}
