package com.sd.business.service.excel.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.IdUtil;
import com.baomidou.dynamic.datasource.toolkit.DynamicDataSourceContextHolder;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.obs.services.ObsClient;
import com.ruoyi.common.core.domain.BaseIdPo;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.sd.business.entity.excel.dto.ExcelGenerateLogDto;
import com.sd.business.entity.excel.dto.ExcelGenerateLogSelectDto;
import com.sd.business.entity.excel.enums.ExcelStatusEnum;
import com.sd.business.entity.excel.enums.ExcelTypeEnum;
import com.sd.business.entity.excel.po.ExcelGenerateLog;
import com.sd.business.entity.excel.vo.ExcelGenerateLogVo;
import com.sd.business.mapper.excel.ExcelGenerateLogMapper;
import com.sd.business.service.excel.ExcelGenerateLogService;
import com.sd.business.strategy.ExcelExportStrategy;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.io.InputStream;
import java.util.Date;
import java.util.StringJoiner;
import java.util.concurrent.Executors;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;


/**
 * <p>
 * excel生成记录 服务实现类
 * </p>
 *
 * @author
 * @since 2023-09-21
 */
@Service
public class ExcelGenerateLogServiceImpl extends ServiceImpl<ExcelGenerateLogMapper, ExcelGenerateLog> implements ExcelGenerateLogService {


    private static final ThreadPoolExecutor excelExecutor = new ThreadPoolExecutor(
            3,
            6,
            60,
            TimeUnit.SECONDS,
            new LinkedBlockingQueue<>(6),
            Executors.defaultThreadFactory(),
            (Runnable runnable, ThreadPoolExecutor executor) -> {
                throw new ServiceException("当前系统导出excel数量过多，系统负载过大，请稍后重试");
            }
    );

    @Value("${obs.ak}")
    private String ak;

    @Value("${obs.sk}")
    private String sk;

    @Value("${obs.endPoint}")
    private String endPoint;

    @Value("${obs.url}")
    private String url;

    @Value("${obs.bucketName}")
    private String bucketName;

    @Value("${spring.profiles.active}")
    private String active;

    @Value("${ruoyi.name}")
    private String name;


    @Override
    public Page<ExcelGenerateLogVo> getPage(ExcelGenerateLogSelectDto dto) {
        IWrapper<ExcelGenerateLog> wrapper = getWrapper();
        wrapper.orderByDesc("egl", ExcelGenerateLog::getId);
        wrapper.eq("egl", ExcelGenerateLog::getCreateUser, SecurityUtils.getUserId());
        wrapper.eq("egl", ExcelGenerateLog::getType, dto.getType());
        Page<ExcelGenerateLogVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        return page;
    }

    @Override
    public ExcelGenerateLogVo detail(Long id) {
        ExcelGenerateLog ExcelGenerateLog = this.getById(id);
        ExcelGenerateLogVo result = BeanUtil.toBean(ExcelGenerateLog, ExcelGenerateLogVo.class);
        return result;
    }

    @Override
    public void add(ExcelGenerateLogDto excelGenerateLogDto) {
        this.save(excelGenerateLogDto);
    }

    @Override
    public void edit(ExcelGenerateLogDto excelGenerateLogDto) {
        this.updateById(excelGenerateLogDto);
    }

    @Override
    public void delete(Long id) {
        this.removeById(id);
    }

    @Override
    public <T> Long generateExcel(ExcelTypeEnum excelTypeEnum, ExcelExportStrategy<T> exportStrategy, String fileName) {

        fileName = fileName + (fileName.endsWith(".xlsx") ? "" : fileName.endsWith(".xls?") ?
                "x" : fileName.endsWith(".") ? "xlsx" : ".xlsx");

        // 文件路径
        String objectKey = new StringJoiner("/")
                .add(name)
                .add(active)
                .add("excel")
                .add(DateUtil.format(new Date(), "yyyy/MM/dd"))
                .add(IdUtil.fastSimpleUUID())
                .add(fileName)
                .toString();

        ExcelGenerateLog excelGenerateLog = new ExcelGenerateLog();
        excelGenerateLog.setType(excelTypeEnum.getType());
        excelGenerateLog.setStatus(ExcelStatusEnum.PROCESSING_DATA.getStatus());
        excelGenerateLog.setExcelName(fileName);
        excelGenerateLog.setObsFileUrl(url + objectKey);
        save(excelGenerateLog);

        Long id = excelGenerateLog.getId();

        excelExecutor.execute(() -> {

            DynamicDataSourceContextHolder.push("business");

            try {
                T data = exportStrategy.getData();
                update(q -> q.eq(BaseIdPo::getId, id).set(ExcelGenerateLog::getStatus, ExcelStatusEnum.GENERATE.getStatus()));

                InputStream inputStream = exportStrategy.getInputStream(data);
                update(q -> q.eq(BaseIdPo::getId, id).set(ExcelGenerateLog::getStatus, ExcelStatusEnum.UPLOAD.getStatus()));

                getObsClient().putObject(bucketName, objectKey, inputStream);
                update(q -> q.eq(BaseIdPo::getId, id)
                        .set(ExcelGenerateLog::getStatus, ExcelStatusEnum.COMPLETE.getStatus())
                        .set(ExcelGenerateLog::getCompletionTime, new Date())
                );
            } catch (Exception e) {
                log.error("导出失败", e);
                update(q -> q.eq(BaseIdPo::getId, id)
                        .set(ExcelGenerateLog::getStatus, ExcelStatusEnum.FAIL.getStatus())
                        .set(ExcelGenerateLog::getCompletionTime, null));
            } finally {
                DynamicDataSourceContextHolder.poll();
            }

        });

        return id;
    }

    private ObsClient getObsClient() {
        return new ObsClient(ak, sk, endPoint);
    }


}
