package com.sd.business.service.in.impl;

import cn.hutool.core.bean.BeanUtil;
import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.sd.business.entity.in.dto.AddAllotDto;
import com.sd.business.entity.in.dto.InOutStorageDto;
import com.sd.business.entity.in.dto.InOutStorageSelectDto;
import com.sd.business.entity.in.emums.InDetailTypeEnum;
import com.sd.business.entity.in.emums.InOutTypeEnum;
import com.sd.business.entity.in.emums.OutDetailTypeEnum;
import com.sd.business.entity.in.po.InOutStorage;
import com.sd.business.entity.in.po.InOutStorageBom;
import com.sd.business.entity.in.vo.InOutStorageVo;
import com.sd.business.entity.warehouse.constant.WarehouseConstant;
import com.sd.business.entity.warehouse.po.Warehouse;
import com.sd.business.mapper.in.InOutStorageMapper;
import com.sd.business.service.in.InOutStorageBomService;
import com.sd.business.service.in.InOutStorageService;
import com.sd.business.service.inventory.InventoryService;
import com.sd.business.service.purchase.PurchaseService;
import com.sd.business.util.CodeEnum;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.List;
import java.util.Objects;


/**
 * <p>
 * 出入库 服务实现类
 * </p>
 *
 * @author
 * @since 2023-07-03
 */
@Service
public class InOutStorageServiceImpl extends ServiceImpl<InOutStorageMapper, InOutStorage> implements InOutStorageService {

    @Autowired
    private InOutStorageBomService inOutStorageBomService;

    @Autowired
    private InventoryService inventoryService;

    @Autowired
    private PurchaseService purchaseService;

    @Override
    public Page<InOutStorageVo> getPage(InOutStorageSelectDto dto) {
        IWrapper<InOutStorage> wrapper = getWrapper();
        wrapper.orderByDesc("ios", InOutStorage::getId);
        wrapper.eq("ios", InOutStorage::getType, dto.getType());
        wrapper.like("ios", InOutStorage::getCode, dto.getCode());
        wrapper.eq("ios", InOutStorage::getDepartmentId, dto.getDepartmentId());
        wrapper.eq("w", Warehouse::getId, dto.getWarehouseId());
        wrapper.eq("ios", InOutStorage::getDetailType, dto.getDetailType());
        wrapper.ge("ios", InOutStorage::getInOutStorageTime, dto.getBeginTime());
        wrapper.le("ios", InOutStorage::getInOutStorageTime, dto.getEndTime());

        return this.baseMapper.getPage(dto.getPage(), wrapper);
    }

    @Override
    public InOutStorageVo detail(Long id) {
        InOutStorage InOutStorage = this.getById(id);
        InOutStorageVo result = BeanUtil.toBean(InOutStorage, InOutStorageVo.class);
        return result;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void add(InOutStorageDto inOutStorageDto) {

        inOutStorageDto.setId(IdWorker.getId());
        List<InOutStorageBom> inOutStorageBomList = inOutStorageDto.getInOutStorageBomList();
        inOutStorageBomList.forEach(item -> item.setInOutStorageId(inOutStorageDto.getId()));

        if (InOutTypeEnum.getInOutType(inOutStorageDto.getType()).equals(InOutTypeEnum.IN)) {
            // 赋值入库单号
            inOutStorageDto.setCode(CodeEnum.IN_CODE.getCode());
            if (InDetailTypeEnum.PURCHASE.getKey().equals(inOutStorageDto.getDetailType())) {
                // 采购入库
                inventoryService.purchaseInStorage(inOutStorageDto.getPurchaseId(), inOutStorageDto.getDepartmentId(), inOutStorageDto.getWarehouseId(), inOutStorageBomList);
                // 更新采购到货数量
                purchaseService.updatePurchaseBomArrivalQuantity(inOutStorageDto.getPurchaseId(), inOutStorageBomList);
            } else {
                // 更新库存
                inventoryService.in(inOutStorageDto.getDepartmentId(), inOutStorageDto.getWarehouseId(), inOutStorageBomList);
            }
        } else {
            // 赋值出库单号
            inOutStorageDto.setCode(CodeEnum.OUT_CODE.getCode());
            // 更新库存
            inventoryService.out(inOutStorageDto.getDepartmentId(), inOutStorageDto.getWarehouseId(), inOutStorageBomList, inOutStorageDto.getLockStorage());
        }
        if (inOutStorageDto.getInOutStorageTime() == null) {
            inOutStorageDto.setInOutStorageTime(new Date());
        }

        // 保存出入库信息
        this.save(inOutStorageDto);
        inOutStorageBomService.saveBatch(inOutStorageBomList);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void addAllot(AddAllotDto dto) {
        long allotFlag = IdWorker.getId();

        // 调出仓库为采购次品仓，调入仓库为半成品仓时，出入库明细类型为生产出库和归还入库
        boolean flag = Objects.equals(dto.getOutWarehouseId(), WarehouseConstant.PURCHASE_DEFECTIVE)
                && Objects.equals(dto.getInWarehouseId(), WarehouseConstant.SEMI_FINISHED_PRODUCT);

        InOutStorageDto outStorageDto = new InOutStorageDto();
        outStorageDto.setType(InOutTypeEnum.OUT.getKey());
        if (flag) {
            outStorageDto.setDetailType(OutDetailTypeEnum.PRODUCTION.getKey());
        } else {
            outStorageDto.setDetailType(OutDetailTypeEnum.ALLOT.getKey());
        }
        outStorageDto.setWarehouseId(dto.getOutWarehouseId());
        outStorageDto.setDepartmentId(dto.getDepartmentId());
        outStorageDto.setApplicant(dto.getApplicant());
        outStorageDto.setRemark(dto.getRemark());
        outStorageDto.setAllotFlag(allotFlag);
        outStorageDto.setInOutStorageBomList(JSON.parseArray(JSON.toJSONString(dto.getInOutStorageBomList()), InOutStorageBom.class));
        add(outStorageDto);

        InOutStorageDto inStorageDto = new InOutStorageDto();
        inStorageDto.setType(InOutTypeEnum.IN.getKey());
        if (flag) {
            inStorageDto.setDetailType(InDetailTypeEnum.GIVE_BACK.getKey());
        } else {
            inStorageDto.setDetailType(InDetailTypeEnum.ALLOT.getKey());
        }
        inStorageDto.setWarehouseId(dto.getInWarehouseId());
        inStorageDto.setDepartmentId(dto.getDepartmentId());
        inStorageDto.setApplicant(dto.getApplicant());
        inStorageDto.setRemark(dto.getRemark());
        inStorageDto.setAllotFlag(allotFlag);
        inStorageDto.setInOutStorageBomList(dto.getInOutStorageBomList());
        add(inStorageDto);

    }

}