package com.sd.business.service.inventory.impl;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.constant.StatusConstant;
import com.ruoyi.common.core.domain.BaseIdPo;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.sd.business.entity.department.constant.DepartmentConstant;
import com.sd.business.entity.department.po.Department;
import com.sd.business.entity.inventory.dto.InventoryFinishedOrderSelectDto;
import com.sd.business.entity.inventory.enums.FinishedOperationTypeEnum;
import com.sd.business.entity.inventory.po.InventoryFinishedOrder;
import com.sd.business.entity.inventory.po.InventoryFinishedOrderDetail;
import com.sd.business.entity.inventory.vo.InventoryFinishedOrderVo;
import com.sd.business.entity.order.po.OrderInfo;
import com.sd.business.entity.order.po.OrderSku;
import com.sd.business.entity.sku.po.SkuSpec;
import com.sd.business.mapper.inventory.InventoryFinishedOrderMapper;
import com.sd.business.service.inventory.InventoryFinishedOrderDetailService;
import com.sd.business.service.inventory.InventoryFinishedOrderService;
import com.sd.business.service.order.OrderService;
import com.sd.framework.util.excel.util.ExcelUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletResponse;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;


/**
 * <p>
 * 成品仓库订单 服务实现类
 * </p>
 *
 * @author
 * @since 2023-10-10
 */
@Service
public class InventoryFinishedOrderServiceImpl extends ServiceImpl<InventoryFinishedOrderMapper, InventoryFinishedOrder> implements InventoryFinishedOrderService {

    @Autowired
    private InventoryFinishedOrderDetailService inventoryFinishedOrderDetailService;

    @Autowired
    private OrderService orderService;

    @Autowired
    private HttpServletResponse response;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void productionWarehousing(List<OrderSku> orderSkuList) {
        // 保存成品订单信息
        List<InventoryFinishedOrder> list = orderSkuList.stream().map(item -> {
            InventoryFinishedOrder inventoryFinishedOrder = new InventoryFinishedOrder();
            inventoryFinishedOrder.setDepartmentId(DepartmentConstant.SD_SPORTS);
            inventoryFinishedOrder.setOrderInfoId(item.getOrderId());
            inventoryFinishedOrder.setOrderSkuId(item.getId());
            inventoryFinishedOrder.setSkuSpecId(item.getSkuSpecId());
            inventoryFinishedOrder.setQuantity(item.getQuantity());
            inventoryFinishedOrder.setStatus(StatusConstant.YES);
            return inventoryFinishedOrder;
        }).collect(Collectors.toList());
        saveBatch(list);

        // 添加流水记录
        inventoryFinishedOrderDetailService.add(list, FinishedOperationTypeEnum.PRODUCTION_WAREHOUSING);
    }

    @Override
    public void removeByOrderId(Long orderId) {
        remove(q -> q.eq(InventoryFinishedOrder::getOrderInfoId, orderId));
        inventoryFinishedOrderDetailService.remove(q -> q.eq(InventoryFinishedOrderDetail::getOrderInfoId, orderId));
    }

    @Override
    public void saleOutOfWarehouse(List<InventoryFinishedOrder> list) {

        List<Long> orderIdList = list.stream()
                .map(InventoryFinishedOrder::getOrderInfoId)
                .distinct()
                .collect(Collectors.toList());

        Map<Long, OrderInfo> orderMap = orderService.mapKEntity(BaseIdPo::getId, q -> q.in(BaseIdPo::getId, orderIdList));

        for (InventoryFinishedOrder item : list) {
            item.setStatus(StatusConstant.NO);
            OrderInfo orderInfo = orderMap.get(item.getOrderInfoId());
            if (Objects.equals(orderInfo.getType(), 2)) {
                item.setDepartmentId(orderInfo.getDepartmentId());
            }
        }

        updateBatchById(list);

        // 添加流水记录
        inventoryFinishedOrderDetailService.add(list, FinishedOperationTypeEnum.SALE_OUT_OF_WAREHOUSE);
    }

    @Override
    public Page<InventoryFinishedOrderVo> getPage(InventoryFinishedOrderSelectDto dto) {
        IWrapper<InventoryFinishedOrder> wrapper = getWrapper();
        wrapper.orderByDesc("ifod", InventoryFinishedOrderDetail::getId);
        wrapper.like("ifod", InventoryFinishedOrderDetail::getCode, dto.getCode());
        wrapper.eq("ss", SkuSpec::getId, dto.getSkuSpecId());
        wrapper.like("ss", SkuSpec::getCode, dto.getSkuSpecCode());
        wrapper.like("ss", SkuSpec::getName, dto.getSkuSpecName());
        wrapper.eq("oi", OrderInfo::getId, dto.getOrderId());
        wrapper.like("oi", OrderInfo::getCode, dto.getOrderCode());
        wrapper.like("oi", OrderInfo::getWlnCode, dto.getOrderWlnCode());
        wrapper.eq("d", Department::getId, dto.getDepartmentId());

        return this.baseMapper.getPage(dto.getPage(), wrapper);
    }

    @Override
    public void excelExport(InventoryFinishedOrderSelectDto dto) {
        dto.setPageNum(1);
        dto.setPageSize(9999999);
        Page<InventoryFinishedOrderVo> page = getPage(dto);
        List<InventoryFinishedOrderVo> records = page.getRecords();

        for (InventoryFinishedOrderVo record : records) {
            record.setOperationTypeStr(record.getOperationType() == 1 ? "入库" : "出库");
        }
        ExcelUtil.export(response, records, InventoryFinishedOrderVo.class);
    }

}
