package com.sd.business.service.order.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.fjhx.file.utils.ObsFileUtil;
import com.ruoyi.common.core.domain.BaseIdPo;
import com.ruoyi.common.core.domain.BasePo;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.ruoyi.framework.mybatis.holder.LogicHolder;
import com.sd.business.entity.bom.po.BomSpec;
import com.sd.business.entity.order.dto.IssueBillSelectDto;
import com.sd.business.entity.order.dto.OrderEncasementDto;
import com.sd.business.entity.order.dto.OrderInfoDto;
import com.sd.business.entity.order.enums.OrderStatusEnum;
import com.sd.business.entity.order.po.OrderEncasement;
import com.sd.business.entity.order.po.OrderEncasementDetail;
import com.sd.business.entity.order.po.OrderInfo;
import com.sd.business.entity.order.po.OrderSku;
import com.sd.business.entity.order.vo.AssemblyDetailVo;
import com.sd.business.entity.order.vo.IssueBillVo;
import com.sd.business.entity.order.vo.OrderEncasementDetailVo;
import com.sd.business.entity.order.vo.OrderEncasementVo;
import com.sd.business.entity.production.dto.MergeOrderDto;
import com.sd.business.mapper.order.IssueBillMapper;
import com.sd.business.service.bom.BomSpecService;
import com.sd.business.service.order.*;
import com.sd.business.service.sku.SkuSpecService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

@Service
public class IssueBillServiceImpl implements IssueBillService {

    @Resource
    private IssueBillMapper mapper;

    @Autowired
    private OrderService orderService;

    @Autowired
    private OrderSkuService orderSkuService;

    @Autowired
    private OrderEncasementService orderEncasementService;

    @Autowired
    private OrderEncasementDetailService orderEncasementDetailService;

    @Autowired
    private SkuSpecService skuSpecService;

    @Autowired
    private BomSpecService bomSpecService;

    @Override
    public Page<IssueBillVo> getPage(IssueBillSelectDto dto) {
        IWrapper<Object> wrapper = IWrapper.getWrapper();
        wrapper.orderByDesc("oi", OrderInfo::getWlnCreateTime);

        wrapper.eq("oi", OrderInfo::getSource, 1);
        wrapper.in("oi", OrderInfo::getStatus, OrderStatusEnum.IN_PRODUCTION.getKey(), OrderStatusEnum.COMPLETION_PRODUCTION.getKey());

        wrapper.like("oi", OrderInfo::getCode, dto.getCode());
        wrapper.eq("oi", OrderInfo::getDepartmentId, dto.getDepartmentId());

        wrapper.ge("ab", OrderInfo::getDeliveryTime, dto.getBeginTime());
        wrapper.le("ab", OrderInfo::getDeliveryTime, dto.getEndTime());

        wrapper.isNull("oi.master_order_id");

        if (dto.getPrintStatus() != null) {
            if (dto.getPrintStatus() == 1) {
                wrapper.isNotNull("oi.wln_print_time");
            } else {
                wrapper.isNull("oi.wln_print_time");
            }
        }

        Page<IssueBillVo> page = mapper.getPage(dto.getPage(), wrapper);
        List<IssueBillVo> records = page.getRecords();

        if (records.isEmpty()) {
            return page;
        }

        // 查询合并订单code
        List<Long> orderIdList = records.stream().map(IssueBillVo::getId).collect(Collectors.toList());
        List<OrderInfo> orderInfoList = orderService.list(q -> q.in(OrderInfo::getMasterOrderId, orderIdList));

        Map<Long, List<String>> map = orderInfoList.stream().collect(
                Collectors.groupingBy(OrderInfo::getMasterOrderId, Collectors.mapping(OrderInfo::getCode, Collectors.toList())));
        records.forEach(item -> item.setGroupOrderCodeList(map.getOrDefault(item.getId(), Collections.emptyList())));

        QueryWrapper<OrderEncasement> orderEncasementWrapper = new QueryWrapper<>();
        orderEncasementWrapper.select(
                "order_id as orderId",
                "count(0) total",
                "ifnull(sum(net_weight),0) totalNetWeight",
                "ifnull(sum(length*width*height/1000000),0) totalVolume"
        );
        orderEncasementWrapper.in("order_id", orderIdList);
        orderEncasementWrapper.groupBy("order_id");

        List<Map<String, Object>> mapList = orderEncasementService.listMaps(orderEncasementWrapper);
        Map<Long, Map<String, Object>> orderEncasementMap = mapList.stream()
                .collect(Collectors.toMap(item -> (Long) item.get("orderId"), Function.identity()));

        for (IssueBillVo record : records) {
            Map<String, Object> itemMap = orderEncasementMap.get(record.getId());
            if (itemMap != null) {
                record.setTotalNetWeight((BigDecimal) itemMap.get("totalNetWeight"));
                record.setTotalVolume((BigDecimal) itemMap.get("totalVolume"));
                record.setTotal((Convert.toInt(itemMap.get("total"))));
            } else {
                record.setTotalNetWeight(BigDecimal.ZERO);
                record.setTotalVolume(BigDecimal.ZERO);
                record.setTotal(0);
            }
        }

        return page;
    }

    @Override
    public void editAddress(OrderInfo orderInfo) {
        orderService.update(q -> q
                .eq(BaseIdPo::getId, orderInfo.getId())
                .set(OrderInfo::getConsignee, orderInfo.getConsignee())
                .set(OrderInfo::getConsigneeNumber, orderInfo.getConsigneeNumber())
                .set(OrderInfo::getProvince, orderInfo.getProvince())
                .set(OrderInfo::getCity, orderInfo.getCity())
                .set(OrderInfo::getCounty, orderInfo.getCounty())
                .set(BasePo::getUpdateTime, new Date())
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
        );
    }

    @Override
    public void editExpressDeliveryId(OrderInfo orderInfo) {
        orderService.update(q -> q
                .eq(BaseIdPo::getId, orderInfo.getId())
                .set(OrderInfo::getExpressDeliveryId, orderInfo.getExpressDeliveryId())
                .set(BasePo::getUpdateTime, new Date())
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
        );
    }

    @Override
    public void editExpressDeliveryCode(OrderEncasement orderEncasement) {
        orderEncasementService.update(q -> q
                .eq(BaseIdPo::getId, orderEncasement.getId())
                .set(OrderEncasement::getTrackingNumber, orderEncasement.getTrackingNumber())
                .set(BasePo::getUpdateTime, new Date())
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
        );
    }

    @Override
    public void editPackagePicture(OrderInfoDto dto) {
        ObsFileUtil.editFile(dto.getPackagePictureList(), dto.getId(), 2);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void merge(MergeOrderDto dto) {
        Long orderId = dto.getId();
        orderService.update(q -> q.eq(OrderInfo::getId, orderId).set(OrderInfo::getMasterOrderId, null));
        orderService.update(q -> q.in(OrderInfo::getId, dto.getMergeIdList()).set(OrderInfo::getMasterOrderId, orderId));
        orderEncasementService.deleteByOrderId(orderId);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void unmerge(Long id) {
        orderService.update(q -> q.eq(OrderInfo::getId, id).set(OrderInfo::getMasterOrderId, null));
        orderService.update(q -> q.eq(OrderInfo::getMasterOrderId, id).set(OrderInfo::getMasterOrderId, null));
        orderEncasementService.deleteByOrderId(id);
    }

    @Override
    public AssemblyDetailVo assemblyDetail(Long orderId) {
        AssemblyDetailVo result = new AssemblyDetailVo();

        List<AssemblyDetailVo.SkuInfo> skuInfoList = getSkuInfoList(orderId);

        List<OrderEncasement> tempOrderEncasementList = orderEncasementService.list(q -> q.eq(OrderEncasement::getOrderId, orderId));
        if (tempOrderEncasementList.size() == 0) {
            result.setSkuInfoList(skuInfoList);
            result.setOrderEncasementList(Collections.emptyList());
            return result;
        }
        List<OrderEncasementVo> orderEncasementList = getOrderEncasementVoList(tempOrderEncasementList, skuInfoList);

        result.setSkuInfoList(skuInfoList);
        result.setOrderEncasementList(orderEncasementList);
        return result;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void deleteAllowance(Long id) {
        orderEncasementService.delete(id);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public AssemblyDetailVo copyAllowance(OrderEncasementDto dto) {

        OrderEncasement orderEncasement = orderEncasementService.getById(dto.getId());
        List<OrderEncasementDetail> list = orderEncasementDetailService.list(q -> q.eq(OrderEncasementDetail::getOrderEncasementId, dto.getId()));

        Integer maxNumber = getMaxNumber(orderEncasement.getOrderId());

        List<OrderEncasement> orderEncasementList = new ArrayList<>();
        List<OrderEncasementDetail> orderEncasementDetailList = new ArrayList<>();

        for (int i = 0; i < dto.getTotal(); i++) {
            maxNumber++;

            OrderEncasement tempOrderEncasement = BeanUtil.copyProperties(orderEncasement, OrderEncasement.class);
            tempOrderEncasement.setId(IdWorker.getId());
            tempOrderEncasement.setNumber(maxNumber);
            tempOrderEncasement.setTrackingNumber(null);

            List<OrderEncasementDetail> tempList = BeanUtil.copyToList(list, OrderEncasementDetail.class);
            for (OrderEncasementDetail orderEncasementDetail : tempList) {
                orderEncasementDetail.setId(IdWorker.getId());
                orderEncasementDetail.setOrderEncasementId(tempOrderEncasement.getId());
            }

            orderEncasementList.add(tempOrderEncasement);
            orderEncasementDetailList.addAll(tempList);
        }

        orderEncasementService.saveBatch(orderEncasementList);
        orderEncasementDetailService.saveBatch(orderEncasementDetailList);

        return verifiedQuantityAndGetDetail(orderEncasement.getOrderId());
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public AssemblyDetailVo addAllowance(OrderEncasementDto dto) {
        dto.setId(IdWorker.getId());
        dto.setNetWeight(BigDecimal.ZERO);
        dto.setNumber(getMaxNumber(dto.getOrderId()) + 1);

        List<OrderEncasementDetail> orderEncasementDetailList = dto.getOrderEncasementDetailList();

        // 查询bom规格净重
        List<Long> bomSpecIdList = orderEncasementDetailList.stream().map(OrderEncasementDetail::getBomSpecId).collect(Collectors.toList());
        Map<Long, BomSpec> bomSpecMap = bomSpecService.mapKEntity(BaseIdPo::getId, q -> q.in(BaseIdPo::getId, bomSpecIdList));

        // 赋值包裹id、包裹净重
        for (OrderEncasementDetail orderEncasementDetail : orderEncasementDetailList) {
            BomSpec bomSpec = bomSpecMap.get(orderEncasementDetail.getBomSpecId());
            BigDecimal netWeight = ObjectUtil.defaultIfNull(bomSpec.getNetWeight(), BigDecimal.ZERO);
            dto.setNetWeight(dto.getNetWeight().add(netWeight.multiply(orderEncasementDetail.getQuantity())));
            orderEncasementDetail.setOrderEncasementId(dto.getId());
        }

        orderEncasementService.save(dto);
        orderEncasementDetailService.saveBatch(orderEncasementDetailList);

        return verifiedQuantityAndGetDetail(dto.getOrderId());
    }

    /**
     * 验证数量并获取明细
     */
    private AssemblyDetailVo verifiedQuantityAndGetDetail(Long orderId) {
        AssemblyDetailVo assemblyDetailVo = assemblyDetail(orderId);
        List<AssemblyDetailVo.SkuInfo> skuInfoList = assemblyDetailVo.getSkuInfoList();
        for (AssemblyDetailVo.SkuInfo skuInfo : skuInfoList) {
            if (skuInfo.getSurplusQuantity().compareTo(BigDecimal.ZERO) < 0) {
                throw new ServiceException("数量不足：装箱数量大于被装箱数量");
            }
        }
        return assemblyDetailVo;
    }

    private List<AssemblyDetailVo.SkuInfo> getSkuInfoList(Long orderId) {

        // 获取合并订单
        List<OrderInfo> mergeOrderList = orderService.list(q -> q.eq(OrderInfo::getMasterOrderId, orderId));

        // 订单id列表
        List<Long> orderIdList = mergeOrderList.stream().map(BaseIdPo::getId).collect(Collectors.toList());
        orderIdList.add(orderId);

        // 赋值产品清单
        List<OrderSku> list = orderSkuService.list(q -> q.in(OrderSku::getOrderId, orderIdList));

        List<AssemblyDetailVo.SkuInfo> skuInfoList = list.stream().map(item -> {
            AssemblyDetailVo.SkuInfo skuInfo = new AssemblyDetailVo.SkuInfo();
            skuInfo.setOrderSkuId(item.getId());
            skuInfo.setSkuSpecId(item.getSkuSpecId());
            skuInfo.setBomSpecId(item.getBomSpecId());
            skuInfo.setTotalQuantity(item.getQuantity());
            skuInfo.setSurplusQuantity(item.getQuantity());
            return skuInfo;
        }).collect(Collectors.toList());

        // 赋值sku规格编号、名称
        skuSpecService.attributeAssign(skuInfoList, AssemblyDetailVo.SkuInfo::getSkuSpecId, (item, skuSpec) -> {
            item.setSkuSpecCode(skuSpec.getCode());
            item.setSkuSpecName(skuSpec.getName());
        });

        return skuInfoList;
    }

    private List<OrderEncasementVo> getOrderEncasementVoList(List<OrderEncasement> tempOrderEncasementList, List<AssemblyDetailVo.SkuInfo> skuInfoList) {

        List<OrderEncasementVo> orderEncasementList = BeanUtil.copyToList(tempOrderEncasementList, OrderEncasementVo.class);
        Map<Long, OrderEncasementVo> orderEncasementVoMap = orderEncasementList.stream().collect(Collectors.toMap(BaseIdPo::getId, Function.identity()));

        // 查询包裹规格sku列表
        List<OrderEncasementDetail> tempOrderEncasementDetailList = orderEncasementDetailService.list(
                q -> q.in(OrderEncasementDetail::getOrderEncasementId, orderEncasementVoMap.keySet()));
        List<OrderEncasementDetailVo> orderEncasementDetailList = BeanUtil.copyToList(tempOrderEncasementDetailList, OrderEncasementDetailVo.class);

        // 设置包裹规格sku品号、品名
        skuSpecService.attributeAssign(orderEncasementDetailList, OrderEncasementDetailVo::getSkuSpecId, (item, skuSpec) -> {
            item.setSkuSpecCode(skuSpec.getCode());
            item.setSkuSpecName(skuSpec.getName());
        });

        // 通过订单id分组
        Map<Long, List<OrderEncasementDetailVo>> map = orderEncasementDetailList.stream()
                .collect(Collectors.groupingBy(OrderEncasementDetail::getOrderEncasementId));
        orderEncasementList.forEach(item -> item.setOrderEncasementDetailList(map.get(item.getId())));

        // 赋值剩余可装箱数量
        Map<Long, List<OrderEncasementDetailVo>> collect = orderEncasementDetailList.stream()
                .collect(Collectors.groupingBy(OrderEncasementDetail::getOrderSkuId));
        for (AssemblyDetailVo.SkuInfo skuInfo : skuInfoList) {
            collect.getOrDefault(skuInfo.getOrderSkuId(), Collections.emptyList()).forEach(item ->
                    skuInfo.setSurplusQuantity(skuInfo.getSurplusQuantity().subtract(item.getQuantity())));
        }

        return orderEncasementList;
    }

    /**
     * 获取包裹最大编号
     */
    private Integer getMaxNumber(Long orderId) {
        LogicHolder.setLogicHolder(true);
        OrderEncasement orderEncasement = orderEncasementService.getOne(q -> q
                .eq(OrderEncasement::getOrderId, orderId)
                .orderByDesc(OrderEncasement::getNumber)
        );
        LogicHolder.clear();

        return orderEncasement == null ? 0 : orderEncasement.getNumber();
    }

}
