package com.sd.business.service.order.impl;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.core.domain.BaseIdPo;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.sd.business.entity.order.dto.OrderEncasementDto;
import com.sd.business.entity.order.dto.OrderEncasementSelectDto;
import com.sd.business.entity.order.po.OrderEncasement;
import com.sd.business.entity.order.po.OrderEncasementDetail;
import com.sd.business.entity.order.vo.OrderEncasementVo;
import com.sd.business.mapper.order.OrderEncasementMapper;
import com.sd.business.service.order.OrderEncasementDetailService;
import com.sd.business.service.order.OrderEncasementService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.stream.Collectors;


/**
 * <p>
 * 订单装箱 服务实现类
 * </p>
 *
 * @author
 * @since 2023-09-25
 */
@Service
public class OrderEncasementServiceImpl extends ServiceImpl<OrderEncasementMapper, OrderEncasement> implements OrderEncasementService {

    @Autowired
    private OrderEncasementDetailService orderEncasementDetailService;

    @Override
    public Page<OrderEncasementVo> getPage(OrderEncasementSelectDto dto) {
        IWrapper<OrderEncasement> wrapper = getWrapper();
        wrapper.orderByDesc("oe", OrderEncasement::getId);
        Page<OrderEncasementVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        return page;
    }

    @Override
    public OrderEncasementVo detail(Long id) {
        OrderEncasement OrderEncasement = this.getById(id);
        OrderEncasementVo result = BeanUtil.toBean(OrderEncasement, OrderEncasementVo.class);
        return result;
    }

    @Override
    public void add(OrderEncasementDto orderEncasementDto) {
        this.save(orderEncasementDto);
    }

    @Override
    public void edit(OrderEncasementDto orderEncasementDto) {
        this.updateById(orderEncasementDto);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void deleteByOrderId(Long orderId) {
        List<OrderEncasement> list = list(q -> q.eq(OrderEncasement::getOrderId, orderId));
        if (list.size() == 0) {
            return;
        }

        List<Long> idList = list.stream().map(BaseIdPo::getId).collect(Collectors.toList());
        removeBatchByIds(idList);
        orderEncasementDetailService.remove(q -> q.in(OrderEncasementDetail::getOrderEncasementId, idList));
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void delete(Long id) {
        removeById(id);
        orderEncasementDetailService.remove(q -> q.eq(OrderEncasementDetail::getOrderEncasementId, id));
    }

}
