package com.sd.business.service.order.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.dynamic.datasource.annotation.DSTransactional;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.file.utils.ObsFileUtil;
import com.ruoyi.common.constant.StatusConstant;
import com.ruoyi.common.core.domain.BaseIdPo;
import com.ruoyi.common.core.domain.BasePo;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.ruoyi.system.utils.UserUtil;
import com.sd.business.entity.department.po.Department;
import com.sd.business.entity.order.dto.OrderExchangeDto;
import com.sd.business.entity.order.dto.OrderExchangeSelectDto;
import com.sd.business.entity.order.enums.OrderClassifyEnum;
import com.sd.business.entity.order.enums.OrderStatusEnum;
import com.sd.business.entity.order.po.*;
import com.sd.business.entity.order.vo.OrderExchangeDetailVo;
import com.sd.business.entity.order.vo.OrderExchangeVo;
import com.sd.business.entity.statement.dto.StatementOfAccountDto;
import com.sd.business.mapper.order.OrderExchangeMapper;
import com.sd.business.service.order.*;
import com.sd.business.service.sku.SkuSpecService;
import com.sd.business.service.statement.StatementOfAccountService;
import com.sd.business.util.CodeEnum;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;


/**
 * <p>
 * 订单退换货 服务实现类
 * </p>
 *
 * @author
 * @since 2023-09-14
 */
@Service
public class OrderExchangeServiceImpl extends ServiceImpl<OrderExchangeMapper, OrderExchange> implements OrderExchangeService {

    @Autowired
    private OrderExchangeDetailService orderExchangeDetailService;

    @Autowired
    private OrderSkuService orderSkuService;

    @Autowired
    private SkuSpecService skuSpecService;

    @Autowired
    private OrderService orderService;

    @Autowired
    private OrderSkuBomService orderSkuBomService;

    @Autowired
    private StatementOfAccountService statementOfAccountService;

    @Override
    public Page<OrderExchangeVo> getPage(OrderExchangeSelectDto dto) {
        IWrapper<OrderExchange> wrapper = getWrapper();
        wrapper.orderByDesc("oe", OrderExchange::getId);
        wrapper.eq("oe", OrderExchange::getOrderInfoId, dto.getOrderInfoId());
        wrapper.like("oe", OrderExchange::getCode, dto.getCode());
        wrapper.like("oi", OrderInfo::getCode, dto.getOrderCode());
        wrapper.like("oi", OrderInfo::getWlnCode, dto.getOrderWlnCode());
        wrapper.eq("d", Department::getId, dto.getDepartmentId());

        Page<OrderExchangeVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        List<OrderExchangeVo> records = page.getRecords();

        if (records.isEmpty()) {
            return page;
        }

        // 赋值操作人名称
        UserUtil.assignmentNickName(records, BasePo::getCreateUser, OrderExchangeVo::setCreateUserName);

        List<Long> idList = records.stream().map(BaseIdPo::getId).collect(Collectors.toList());
        List<OrderExchangeDetail> list = orderExchangeDetailService.list(q -> q.in(OrderExchangeDetail::getOrderExchangeId, idList));
        List<OrderExchangeDetailVo> orderExchangeDetailList = BeanUtil.copyToList(list, OrderExchangeDetailVo.class);

        // 赋值sku规格id和采购数量
        orderSkuService.attributeAssign(orderExchangeDetailList, OrderExchangeDetailVo::getOrderSkuId, (item, orderSku) -> {
            item.setSkuSpecId(orderSku.getSkuSpecId());
            item.setBuyQuantity(orderSku.getQuantity());
            item.setReturnAmount((orderSku.getUnitPrice()
                    .add(orderSku.getCustomProcessingFee())
                    .add(orderSku.getLssueFee())
                    .add(orderSku.getDeliveryMaterialsFee())
                    .add(orderSku.getPackingLabor())
                    .add(orderSku.getPackagingMaterialCost())
                    .add(orderSku.getManagementFee())).multiply(item.getQuantity()));
        });

        // 赋值sku品号品名
        skuSpecService.attributeAssign(orderExchangeDetailList, OrderExchangeDetailVo::getSkuSpecId, (item, skuSpec) -> {
            item.setSkuSpecCode(skuSpec.getCode());
            item.setSkuSpecName(skuSpec.getName());
        });

        Map<Long, List<OrderExchangeDetailVo>> map = orderExchangeDetailList.stream()
                .collect(Collectors.groupingBy(OrderExchangeDetailVo::getOrderExchangeId));

        for (OrderExchangeVo record : records) {
            record.setOrderExchangeDetailList(map.getOrDefault(record.getId(), Collections.emptyList()));
        }

        return page;
    }

    @Override
    public OrderExchangeVo detail(Long id) {
        OrderExchange orderExchange = this.getById(id);
        if (orderExchange == null) {
            throw new ServiceException("未知订单号");
        }

        OrderExchangeVo result = BeanUtil.toBean(orderExchange, OrderExchangeVo.class);
        List<OrderExchangeDetail> list = orderExchangeDetailService.list(q -> q.eq(OrderExchangeDetail::getOrderExchangeId, id));
        List<OrderExchangeDetailVo> orderExchangeDetailList = BeanUtil.copyToList(list, OrderExchangeDetailVo.class);

        // 赋值sku规格id和采购数量
        orderSkuService.attributeAssign(orderExchangeDetailList, OrderExchangeDetailVo::getOrderSkuId, (item, orderSku) -> {
            item.setSkuSpecId(orderSku.getSkuSpecId());
            item.setBuyQuantity(orderSku.getQuantity());
        });

        // 赋值sku品号品名
        skuSpecService.attributeAssign(orderExchangeDetailList, OrderExchangeDetailVo::getSkuSpecId, (item, skuSpec) -> {
            item.setSkuSpecCode(skuSpec.getCode());
            item.setSkuSpecName(skuSpec.getName());
        });

        result.setOrderExchangeDetailList(orderExchangeDetailList);

        return result;
    }

    @DSTransactional
    @Override
    public void add(OrderExchangeDto orderExchangeDto) {
        orderExchangeDto.setId(IdWorker.getId());

        List<OrderExchangeDetail> orderExchangeDetailList = orderExchangeDto.getOrderExchangeDetailList();

        // 获取订单sku id集合
        List<Long> orderSkuIdList = orderExchangeDetailList.stream()
                .map(OrderExchangeDetail::getOrderSkuId)
                .collect(Collectors.toList());

        // 根据集合查询订单数量
        Map<Long, BigDecimal> map = orderSkuService.mapKV(BaseIdPo::getId, OrderSku::getQuantity,
                q -> q.in(BaseIdPo::getId, orderSkuIdList));

        // 已添加订单产品退货列表
        List<OrderExchangeDetail> list = new ArrayList<>();
        if (orderExchangeDto.getType().equals(1)) {
            list = orderExchangeDetailService.list(q -> q.in(OrderExchangeDetail::getOrderSkuId, orderSkuIdList));
        }

        for (OrderExchangeDetail orderExchangeDetail : orderExchangeDetailList) {

            Long orderSkuId = orderExchangeDetail.getOrderSkuId();

            // 数量为空，没找到订单sku id
            BigDecimal quantity = map.get(orderSkuId);
            if (quantity == null) {
                throw new ServiceException("未知订单skuId：" + orderSkuId);
            }

            // 退换货数量大于订单产品购买数量
            if (orderExchangeDetail.getQuantity().compareTo(quantity) > 0) {
                throw new ServiceException("退换货数量不能大于订单产品购买数量");
            }

            // 如果退货，已退货数量+本次退货数量不能大于订单产品数量
            if (orderExchangeDto.getType().equals(1)) {
                BigDecimal returnQuantity = list.stream()
                        .filter(item -> Objects.equals(item.getOrderSkuId(), orderSkuId))
                        .map(OrderExchangeDetail::getQuantity)
                        .reduce(BigDecimal.ZERO, BigDecimal::add);

                if (returnQuantity.add(orderExchangeDetail.getQuantity()).compareTo(quantity) > 0) {
                    throw new ServiceException("已记录退货数量加本次退货数量大于订单产品购买数量");
                }

            }

            orderExchangeDetail.setOrderExchangeId(orderExchangeDto.getId());
            orderExchangeDetail.setReturnStatus(StatusConstant.NO);
            orderExchangeDetail.setExchangeStatus(StatusConstant.NO);
        }

        orderExchangeDto.setCode(orderExchangeDto.getType() == 1 ? CodeEnum.TH_CODE.getCode() : CodeEnum.HH_CODE.getCode());
        save(orderExchangeDto);
        orderExchangeDetailService.saveBatch(orderExchangeDetailList);
        ObsFileUtil.saveFile(orderExchangeDto.getFileList(), orderExchangeDto.getId());

        Map<Long, BigDecimal> exchangeDetailMap = orderExchangeDetailList.stream()
                .filter(item -> item.getQuantity().compareTo(BigDecimal.ZERO) > 0)
                .collect(Collectors.toMap(OrderExchangeDetail::getOrderSkuId, OrderExchangeDetail::getQuantity));
        if (ObjectUtil.isEmpty(exchangeDetailMap)) {
            return;
        }
        // 新增售后订单，分类修改为售后订单
        OrderInfo orderInfo = orderService.getById(orderExchangeDto.getOrderInfoId());
        orderInfo.setId(IdWorker.getId());
        orderInfo.setWlnUid(null);
        orderInfo.setClassify(OrderClassifyEnum.AFTER_SALE_ORDER.getKey());
        orderInfo.setStatus(OrderStatusEnum.EXCHANGE.getKey());
        orderInfo.setTotalAmount(BigDecimal.ZERO);
        orderInfo.setProductTotalAmount(BigDecimal.ZERO);
        orderInfo.setCustomProcessingFee(BigDecimal.ZERO);
        orderInfo.setLssueFee(BigDecimal.ZERO);
        orderInfo.setDeliveryMaterialsFee(BigDecimal.ZERO);
        orderInfo.setPackingLabor(BigDecimal.ZERO);
        orderInfo.setManagementFee(BigDecimal.ZERO);
        orderInfo.setPackagingMaterialCost(BigDecimal.ZERO);
        List<OrderSku> orderSkuList = orderSkuService.list(q -> q.in(BaseIdPo::getId, exchangeDetailMap.keySet()));
        Map<Long, Long> orderSkuIdMap = new HashMap<>();
        for (OrderSku orderSku : orderSkuList) {
            BigDecimal quantity = exchangeDetailMap.get(orderSku.getId());
            long orderSkuId = IdWorker.getId();
            orderSkuIdMap.put(orderSku.getId(), orderSkuId);
            orderSku.setQuantity(quantity);
            orderSku.setId(orderSkuId);
            orderSku.setOrderId(orderInfo.getId());
            this.addOrderInfoPrice(orderInfo, orderSku);
        }
        List<OrderSkuBom> orderSkuBomList = orderSkuBomService.list(q -> q.in(OrderSkuBom::getOrderSkuId, exchangeDetailMap.keySet()));
        // 替换订单id和订单sku id
        for (OrderSkuBom orderSkuBom : orderSkuBomList) {
            orderSkuBom.setId(null);
            orderSkuBom.setOrderId(orderInfo.getId());
            orderSkuBom.setOrderSkuId(orderSkuIdMap.get(orderSkuBom.getOrderSkuId()));
        }

        // 新增订单信息
        orderService.save(orderInfo);
        orderSkuService.saveBatch(orderSkuList);
        orderSkuBomService.saveBatch(orderSkuBomList);
        // 生成对账单
        StatementOfAccountDto statement = new StatementOfAccountDto();
        statement.setDepartmentId(orderInfo.getDepartmentId());
        statement.setOrderIdList(Collections.singletonList(orderInfo.getId()));
        statementOfAccountService.add(statement);
    }

    @Override
    public void delete(Long id) {
        this.removeById(id);
    }

    /**
     * 添加订单价格
     */
    private void addOrderInfoPrice(OrderInfo orderInfo, OrderSku orderSku) {
        BigDecimal quantity = orderSku.getQuantity();

        BigDecimal productTotalAmount = quantity.multiply(orderSku.getUnitPrice());
        BigDecimal customProcessingFee = quantity.multiply(orderSku.getCustomProcessingFee());
        BigDecimal lssueFee = quantity.multiply(orderSku.getLssueFee());
        BigDecimal deliveryMaterialsFee = quantity.multiply(orderSku.getDeliveryMaterialsFee());
        BigDecimal packingLabor = quantity.multiply(orderSku.getPackingLabor());
        BigDecimal packagingMaterialCost = quantity.multiply(orderSku.getPackagingMaterialCost());
        BigDecimal managementFee = quantity.multiply(orderSku.getManagementFee());
        BigDecimal totalAmount = productTotalAmount
                .add(customProcessingFee)
                .add(lssueFee)
                .add(deliveryMaterialsFee)
                .add(packingLabor)
                .add(packagingMaterialCost)
                .add(managementFee);

        orderInfo.setProductTotalAmount(orderInfo.getProductTotalAmount().add(productTotalAmount));
        orderInfo.setCustomProcessingFee(orderInfo.getCustomProcessingFee().add(customProcessingFee));
        orderInfo.setLssueFee(orderInfo.getLssueFee().add(lssueFee));
        orderInfo.setDeliveryMaterialsFee(orderInfo.getDeliveryMaterialsFee().add(deliveryMaterialsFee));
        orderInfo.setPackingLabor(orderInfo.getPackingLabor().add(packingLabor));
        orderInfo.setManagementFee(orderInfo.getManagementFee().add(managementFee));
        orderInfo.setPackagingMaterialCost(orderInfo.getPackagingMaterialCost().add(packagingMaterialCost));
        orderInfo.setTotalAmount(orderInfo.getTotalAmount().add(totalAmount));
    }
}
