package com.sd.business.service.production.impl;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.core.domain.BaseSelectDto;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.sd.business.entity.artwork.po.ArtworkLibrary;
import com.sd.business.entity.bom.po.BomSpec;
import com.sd.business.entity.order.enums.OrderStatusEnum;
import com.sd.business.entity.order.po.OrderInfo;
import com.sd.business.entity.order.po.OrderSku;
import com.sd.business.entity.production.dto.ProductionWorkOrderSelectDto;
import com.sd.business.entity.production.po.ProductionWorkOrder;
import com.sd.business.entity.production.vo.ProductionOrderScheduleVo;
import com.sd.business.entity.production.vo.ProductionWorkOrderVo;
import com.sd.business.entity.sku.po.SkuSpec;
import com.sd.business.mapper.production.ProductionWorkOrderMapper;
import com.sd.business.service.artwork.ArtworkLibraryService;
import com.sd.business.service.inventory.InventoryFinishedService;
import com.sd.business.service.order.OrderService;
import com.sd.business.service.order.OrderSkuService;
import com.sd.business.service.production.ProductionWorkOrderService;
import com.sd.business.service.sku.SkuSpecService;
import com.sd.business.util.CodeEnum;
import com.sd.mq.config.WorkOrderConfig;
import com.sd.mq.entity.CombinedWorkOrderMessage;
import com.sd.mq.entity.WorkOrderDetail;
import com.sd.mq.entity.WorkOrderMessage;
import com.sd.mq.enums.WorkOrderStatusEnum;
import com.sd.mq.util.RabbitMqUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 生产工单 服务实现类
 * </p>
 *
 * @author
 * @since 2023-07-19
 */
@Service
public class ProductionWorkOrderServiceImpl extends ServiceImpl<ProductionWorkOrderMapper, ProductionWorkOrder> implements ProductionWorkOrderService {

    @Autowired
    private OrderService orderService;

    @Autowired
    private OrderSkuService orderSkuService;

    @Autowired
    private SkuSpecService skuSpecService;

    @Autowired
    private ArtworkLibraryService artworkLibraryService;

    @Autowired
    private InventoryFinishedService inventoryFinishedService;

    @Override
    public Map<Long, List<ProductionOrderScheduleVo>> getProductionOrderScheduleMap(List<Long> orderIdList) {
        if (orderIdList.isEmpty()) {
            return Collections.emptyMap();
        }

        IWrapper<Object> wrapper = IWrapper.getWrapper();
        wrapper.in("pwo", ProductionWorkOrder::getOrderId, orderIdList);
        wrapper.ne("pwo", ProductionWorkOrder::getStatus, WorkOrderStatusEnum.RESCHEDULE.getKey());
        wrapper.groupBy("pwo.order_sku_id");
        List<ProductionOrderScheduleVo> list = baseMapper.getProductionOrderScheduleList(wrapper);

        return list.stream().collect(Collectors.groupingBy(ProductionOrderScheduleVo::getOrderId));
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void add(Long orderId) {
        OrderInfo orderInfo = orderService.getById(orderId);
        if (orderInfo == null) {
            throw new ServiceException("未知订单");
        }

        List<ProductionWorkOrder> productionWorkOrderList = new ArrayList<>();

        List<OrderSku> orderSkuList = orderSkuService.list(q -> q.eq(OrderSku::getOrderId, orderId));

        String code = ObjectUtil.defaultIfNull(orderInfo.getWlnCode(), orderInfo.getCode());

        int num = 1;

        for (OrderSku orderSku : orderSkuList) {
            BigDecimal quantity = orderSku.getQuantity();
            for (int i = 0; i < quantity.intValue(); i++) {
                ProductionWorkOrder productionWorkOrder = new ProductionWorkOrder();
                productionWorkOrder.setType(1);
                productionWorkOrder.setCode(code + String.format("-%05d", num));
                productionWorkOrder.setOrderId(orderId);
                productionWorkOrder.setOrderSkuId(orderSku.getId());
                productionWorkOrder.setSkuSpecId(orderSku.getSkuSpecId());
                productionWorkOrder.setBomSpecId(orderSku.getBomSpecId());
                productionWorkOrder.setPrintType(orderSku.getPrintType());
                productionWorkOrder.setStatus(0);
                productionWorkOrderList.add(productionWorkOrder);
                num++;
            }
        }

        skuSpecService.attributeAssign(productionWorkOrderList, ProductionWorkOrder::getSkuSpecId, (item, skuSpec) -> {
            item.setMachinedPanel(skuSpec.getMachinedPanel());
        });

        saveBatch(productionWorkOrderList);

        sendMq(productionWorkOrderList);
    }

    @Override
    public void sendMq(List<ProductionWorkOrder> list) {
        if (list.isEmpty()) {
            return;
        }

        // 订单详情
        List<Long> orderSkuIdList = list.stream().map(ProductionWorkOrder::getOrderSkuId).collect(Collectors.toList());
        Map<Long, OrderSku> orderSkuMap = orderSkuService.byIdsToMap(orderSkuIdList);

        // 图稿库
        List<Long> artworkLibraryIdList = orderSkuMap.values().stream()
                .map(OrderSku::getArtworkLibraryId)
                .filter(Objects::nonNull)
                .filter(item -> item != 0)
                .distinct()
                .collect(Collectors.toList());
        Map<Long, ArtworkLibrary> artworkLibraryMap;
        if (artworkLibraryIdList.isEmpty()) {
            artworkLibraryMap = Collections.emptyMap();
        } else {
            artworkLibraryMap = artworkLibraryService.byIdsToMap(artworkLibraryIdList);
        }

        // sku规格
        List<Long> skuSpecIdList = list.stream().map(ProductionWorkOrder::getSkuSpecId).collect(Collectors.toList());
        Map<Long, SkuSpec> skuSpecMap = skuSpecService.byIdsToMap(skuSpecIdList);

        List<WorkOrderDetail> workOrderDetailList = list.stream().map(item -> {
            WorkOrderDetail workOrderDetail = new WorkOrderDetail();

            workOrderDetail.setWorkOrderId(item.getId());

            Long orderSkuId = item.getOrderSkuId();
            OrderSku orderSku = orderSkuMap.get(orderSkuId);
            Long artworkLibraryId = orderSku.getArtworkLibraryId();

            if (artworkLibraryId != null && artworkLibraryId != 0) {
                ArtworkLibrary artworkLibrary = artworkLibraryMap.get(artworkLibraryId);
                if (artworkLibrary != null) {
                    workOrderDetail.setImgPath("artwork\\" + artworkLibrary.getArtworkName() + "\\" + artworkLibrary.getImgName());
                    workOrderDetail.setFilePath("artwork\\" + artworkLibrary.getArtworkName() + "\\" + artworkLibrary.getFileName());
                }
            }

            if (StrUtil.isAllBlank(workOrderDetail.getImgPath(), workOrderDetail.getFilePath())) {
                workOrderDetail.setImgPath("tempArtwork\\" + orderSku.getBlueprint().replace("https://os.winfaster.cn/sd/", "")
                        .replace("test/", "")
                        .replace("prod/", "")
                        .replace("dev/", ""));
                workOrderDetail.setFilePath("tempArtwork\\" + orderSku.getProductionDocument().replace("https://os.winfaster.cn/sd/", "")
                        .replace("test/", "")
                        .replace("prod/", "")
                        .replace("dev/", ""));
            }

            SkuSpec skuSpec = skuSpecMap.get(item.getSkuSpecId());

            workOrderDetail.setLength(skuSpec.getLength());
            workOrderDetail.setWide(skuSpec.getWidth());
            workOrderDetail.setHight(skuSpec.getHeight());

            return workOrderDetail;
        }).collect(Collectors.toList());

        WorkOrderMessage workOrderMessage = new WorkOrderMessage();
        workOrderMessage.setWorkOrderDetailList(workOrderDetailList);

        RabbitMqUtil.send(WorkOrderConfig.DIRECT_EXCHANGE_NAME, WorkOrderConfig.ADD_WORK_ORDER_QUEUE_NAME, workOrderMessage);

    }

    @Override
    public Page<ProductionWorkOrderVo> getPage(ProductionWorkOrderSelectDto dto) {
        IWrapper<ProductionWorkOrder> wrapper = getWrapper();
        wrapper.orderByDesc("pwo", ProductionWorkOrder::getId);
        wrapper.isNull("pwo.master_id");

        wrapper.in("pwo", ProductionWorkOrder::getStatus, dto.getStatus());

        wrapper.like("oi", OrderInfo::getCode, dto.getOrderCode());
        wrapper.like("oi", OrderInfo::getWlnCode, dto.getOrderWlnCode());

        wrapper.like("pwo", ProductionWorkOrder::getCode, dto.getProductionWorkOrderCode());

        wrapper.like("ss", SkuSpec::getCode, dto.getSkuSpecCode());
        wrapper.like("ss", SkuSpec::getName, dto.getSkuSpecName());

        wrapper.like("bs", BomSpec::getCode, dto.getBomSpecCode());
        wrapper.like("bs", BomSpec::getName, dto.getBomSpecName());

        return this.baseMapper.getPage(dto.getPage(), wrapper);
    }

    /**
     * 查询订单所有工单是否完成，若完成，修改订单状态为已完成
     */
    @Override
    public void completeOrder(Long orderId) {

        // 存在待投产、已扫描、生产中、生产异常的工单，则订单位完成
        long count = count(q -> q
                .in(ProductionWorkOrder::getStatus,
                        WorkOrderStatusEnum.TO_BE_PRODUCED.getKey(),
                        WorkOrderStatusEnum.SCANNED.getKey(),
                        WorkOrderStatusEnum.IN_PRODUCTION.getKey(),
                        WorkOrderStatusEnum.PRODUCTION_ANOMALY.getKey())
                .last("limit 1")
        );

        if (count != 0) {
            return;
        }

        // 订单已完成或已发货跳过
        OrderInfo orderInfo = orderService.getById(orderId);
        if (Objects.equals(orderInfo.getStatus(), OrderStatusEnum.COMPLETION_PRODUCTION.getKey())
                || Objects.equals(orderInfo.getStatus(), OrderStatusEnum.HAVE_BEEN_SHIPPED.getKey())) {
            return;
        }

        // 生产入库
        inventoryFinishedService.productionWarehousing(Collections.singletonList(orderId));

        // 更新订单状态已完成
        orderInfo.setStatus(OrderStatusEnum.COMPLETION_PRODUCTION.getKey());
        orderService.updateById(orderInfo);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void complete(BaseSelectDto dto) {

        ProductionWorkOrder productionWorkOrder = getById(dto.getId());
        productionWorkOrder.setCompleteTime(new Date());
        productionWorkOrder.setStatus(WorkOrderStatusEnum.PRODUCTION_COMPLETION.getKey());
        updateById(productionWorkOrder);

        // 单片
        if (productionWorkOrder.getType() == 1) {
            completeOrder(productionWorkOrder.getOrderId());
        }

        // 批量
        else {
            List<ProductionWorkOrder> list = list(q -> q.eq(ProductionWorkOrder::getMasterId, dto.getId()));
            for (ProductionWorkOrder item : list) {
                item.setCompleteTime(new Date());
                item.setStatus(WorkOrderStatusEnum.PRODUCTION_COMPLETION.getKey());
            }
            updateBatchById(list);

            List<Long> orderIdList = list.stream().map(ProductionWorkOrder::getOrderId).distinct().collect(Collectors.toList());
            for (Long orderId : orderIdList) {
                completeOrder(orderId);
            }
        }

    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public synchronized void combined(List<Long> idList) {
        if (idList.isEmpty()) {
            return;
        }

        List<ProductionWorkOrder> productionWorkOrderList = listByIds(idList);

        if (idList.size() != productionWorkOrderList.size()) {
            throw new ServiceException("存在未知工单");
        }

        ProductionWorkOrder temp = productionWorkOrderList.get(0);
        for (ProductionWorkOrder productionWorkOrder : productionWorkOrderList) {
            if (Objects.equals(temp.getType(), 2)) {
                throw new ServiceException("批量工单无法合并");
            }
            if (productionWorkOrder.getMasterId() != null) {
                throw new ServiceException("存在已合并的工单");
            }
            if (!Objects.equals(temp.getSkuSpecId(), productionWorkOrder.getSkuSpecId())) {
                throw new ServiceException("存在不同sku");
            }
            if (!Objects.equals(temp.getBomSpecId(), productionWorkOrder.getBomSpecId())) {
                throw new ServiceException("存在不同bom");
            }
            if (!Objects.equals(productionWorkOrder.getStatus(), WorkOrderStatusEnum.TO_BE_PRODUCED.getKey())) {
                throw new ServiceException("存在工单不在待投产状态");
            }
        }

        // 创建批量工单
        ProductionWorkOrder productionWorkOrder = getProductionWorkOrder(temp);
        save(productionWorkOrder);

        // 单片工单关联批量工单
        productionWorkOrderList.forEach(item -> item.setMasterId(productionWorkOrder.getId()));
        updateBatchById(productionWorkOrderList);

        // 发送mq
        RabbitMqUtil.send(WorkOrderConfig.DIRECT_EXCHANGE_NAME, WorkOrderConfig.COMBINED_WORK_ORDER_QUEUE_NAME,
                new CombinedWorkOrderMessage(productionWorkOrder.getId(), idList));

    }

    private ProductionWorkOrder getProductionWorkOrder(ProductionWorkOrder temp) {
        String code = CodeEnum.BATH_PRODUCTION_WORK_ORDER_CODE.getCode();
        ProductionWorkOrder productionWorkOrder = new ProductionWorkOrder();
        productionWorkOrder.setCode(code);
        productionWorkOrder.setSkuSpecId(temp.getSkuSpecId());
        productionWorkOrder.setBomSpecId(temp.getBomSpecId());
        productionWorkOrder.setPrintType(temp.getPrintType());
        productionWorkOrder.setMachinedPanel(temp.getMachinedPanel());
        productionWorkOrder.setStatus(0);
        productionWorkOrder.setType(2);
        return productionWorkOrder;
    }

}
