package com.sd.business.service.sku.impl;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.sd.business.entity.sku.dto.SkuDefaultPackageDto;
import com.sd.business.entity.sku.dto.SkuDefaultPackageSelectDto;
import com.sd.business.entity.sku.po.Sku;
import com.sd.business.entity.sku.po.SkuDefaultPackage;
import com.sd.business.entity.sku.vo.SkuDefaultPackageVo;
import com.sd.business.mapper.sku.SkuDefaultPackageMapper;
import com.sd.business.service.sku.SkuDefaultPackageService;
import com.sd.business.service.sku.SkuService;
import com.sd.framework.util.Assert;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;


/**
 * <p>
 * sku默认包材配置 服务实现类
 * </p>
 *
 * @author
 * @since 2023-11-08
 */
@Service
public class SkuDefaultPackageServiceImpl extends ServiceImpl<SkuDefaultPackageMapper, SkuDefaultPackage> implements SkuDefaultPackageService {

    @Autowired
    private SkuService skuService;

    @Override
    public Page<SkuDefaultPackageVo> getPage(SkuDefaultPackageSelectDto dto) {
        IWrapper<SkuDefaultPackage> wrapper = getWrapper();
        wrapper.orderByDesc("sdp", SkuDefaultPackage::getId);
        wrapper.like("s", Sku::getCode, dto.getSkuCode());
        return this.baseMapper.getPage(dto.getPage(), wrapper);
    }

    @Override
    public SkuDefaultPackageVo detail(Long id) {
        SkuDefaultPackage skuDefaultPackage = this.getById(id);
        Assert.notNull(skuDefaultPackage, "未知配置");

        SkuDefaultPackageVo result = BeanUtil.toBean(skuDefaultPackage, SkuDefaultPackageVo.class);

        Sku sku = skuService.getById(skuDefaultPackage.getSkuId());
        Assert.notNull(sku, "未知sku");

        result.setSkuCode(sku.getCode());
        return result;
    }

    @Override
    public void add(SkuDefaultPackageDto skuDefaultPackageDto) {
        this.save(skuDefaultPackageDto);
    }

    @Override
    public void edit(SkuDefaultPackageDto skuDefaultPackageDto) {
        this.updateById(skuDefaultPackageDto);
    }

    @Override
    public void delete(Long id) {
        this.removeById(id);
    }

}
