package com.sd.business.service.sku.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.tenant.entity.dict.po.DictCommonData;
import com.fjhx.tenant.service.dict.DictCommonDataService;
import com.ruoyi.common.core.domain.BaseIdPo;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.sd.business.entity.board.dto.SkuSpecQuotationDto;
import com.sd.business.entity.board.vo.SkuSpecQuotationVo;
import com.sd.business.entity.bom.bo.BomSpecBo;
import com.sd.business.entity.bom.po.Bom;
import com.sd.business.entity.bom.po.BomSpec;
import com.sd.business.entity.department.po.Department;
import com.sd.business.entity.inventory.po.Inventory;
import com.sd.business.entity.price.po.PriceBillingStandard;
import com.sd.business.entity.price.po.PriceBillingStandardDetail;
import com.sd.business.entity.sku.bo.SkuSpecBo;
import com.sd.business.entity.sku.dto.SkuSpecDto;
import com.sd.business.entity.sku.dto.SkuSpecSelectDto;
import com.sd.business.entity.sku.po.Sku;
import com.sd.business.entity.sku.po.SkuSpec;
import com.sd.business.entity.sku.po.SkuSpecLink;
import com.sd.business.entity.sku.vo.SkuSpecVo;
import com.sd.business.entity.warehouse.constant.WarehouseConstant;
import com.sd.business.mapper.sku.SkuSpecMapper;
import com.sd.business.service.department.DepartmentService;
import com.sd.business.service.inventory.InventoryService;
import com.sd.business.service.price.PriceBillingStandardDetailService;
import com.sd.business.service.price.PriceBillingStandardService;
import com.sd.business.service.sku.SkuService;
import com.sd.business.service.sku.SkuSpecLinkService;
import com.sd.business.service.sku.SkuSpecService;
import com.sd.framework.util.Assert;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;


/**
 * <p>
 * sku规格 服务实现类
 * </p>
 *
 * @author
 * @since 2023-07-05
 */
@Service
public class SkuSpecServiceImpl extends ServiceImpl<SkuSpecMapper, SkuSpec> implements SkuSpecService {

    @Autowired
    private InventoryService inventoryService;

    @Autowired
    private DepartmentService departmentService;

    @Autowired
    private SkuService skuService;

    @Autowired
    private SkuSpecLinkService skuSpecLinkService;

    @Autowired
    private PriceBillingStandardService priceBillingStandardService;

    @Autowired
    private PriceBillingStandardDetailService priceBillingStandardDetailService;

    @Autowired
    private DictCommonDataService dictCommonDataService;

    @Override
    public Page<SkuSpecVo> getPage(SkuSpecSelectDto dto) {
        IWrapper<SkuSpec> wrapper = getWrapper();
        wrapper.orderByDesc("ss", SkuSpec::getId);
        Page<SkuSpecVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        return page;
    }

    @Override
    public SkuSpecVo detail(Long id) {
        SkuSpec SkuSpec = this.getById(id);
        SkuSpecVo result = BeanUtil.toBean(SkuSpec, SkuSpecVo.class);
        return result;
    }

    @Override
    public void add(SkuSpecDto skuSpecDto) {
        this.save(skuSpecDto);
    }

    @Override
    public void edit(SkuSpecDto skuSpecDto) {
        this.updateById(skuSpecDto);
    }

    @Override
    public void delete(Long id) {
        this.removeById(id);
    }

    @Override
    public Map<Long, BomSpecBo> getBomSpecBoByIdList(Collection<Long> bomSpecIdList) {

        if (ObjectUtil.isEmpty(bomSpecIdList)) {
            return Collections.emptyMap();
        }

        bomSpecIdList = bomSpecIdList.stream().filter(ObjectUtil::isNotNull).distinct().collect(Collectors.toList());

        if (ObjectUtil.isEmpty(bomSpecIdList)) {
            return Collections.emptyMap();
        }

        List<BomSpecBo> list = baseMapper.getBomSpecBoList(Wrappers.query().in("bs.id", bomSpecIdList));

        return list.stream().collect(Collectors.toMap(BomSpecBo::getBomSpecId, Function.identity()));
    }

    @Override
    public BomSpecBo getBomSpecBoById(Long bomSpecId) {
        Map<Long, BomSpecBo> map = getBomSpecBoByIdList(Collections.singletonList(bomSpecId));
        return map.get(bomSpecId);
    }

    @Override
    public BigDecimal getSkuInventoryQuantity(Long id) {
        SkuSpec skuSpec = this.getById(id);
        if (skuSpec == null) {
            throw new ServiceException("没有找到sku信息");
        }
        Long bomSpecId = skuSpec.getBomSpecId();
        if (ObjectUtil.isEmpty(bomSpecId)) {
            return BigDecimal.ZERO;
        }
        Map<Long, BomSpecBo> bomSpecBoMap = this.getBomSpecBoByIdList(Collections.singleton(bomSpecId));
        BomSpecBo bomSpecBo = bomSpecBoMap.get(bomSpecId);
        if (bomSpecBo == null) {
            throw new ServiceException("未知bom规格id：" + bomSpecId);
        }
        Long warehouseId;
        // 主材
        if (ObjectUtil.equals(bomSpecBo.getClassifyParentId(), 1L)) {
            warehouseId = WarehouseConstant.SEMI_FINISHED_PRODUCT;
        } else {
            warehouseId = WarehouseConstant.PACKAGING_MATERIAL;
        }
        Inventory inventory = inventoryService.getOne(q -> q
                .eq(Inventory::getWarehouseId, warehouseId)
                .eq(Inventory::getBomSpecId, bomSpecId));
        return inventory == null ? BigDecimal.ZERO : inventory.getQuantity();
    }

    @Override
    public Map<Long, SkuSpecBo> getSkuSpecBoByIdList(Collection<Long> skuSpecIdList) {

        if (ObjectUtil.isEmpty(skuSpecIdList)) {
            return Collections.emptyMap();
        }

        skuSpecIdList = skuSpecIdList.stream().filter(ObjectUtil::isNotNull).distinct().collect(Collectors.toList());

        if (ObjectUtil.isEmpty(skuSpecIdList)) {
            return Collections.emptyMap();
        }

        List<SkuSpecBo> list = baseMapper.getSkuSpecBoList(Wrappers.query().in("ss.id", skuSpecIdList));

        return list.stream().collect(Collectors.toMap(SkuSpecBo::getSkuSpecId, Function.identity()));

    }

    @Override
    public List<SkuSpecQuotationVo> getSkuSpecQuotationList(SkuSpecQuotationDto dto) {
        Assert.notNull(dto.getDepartmentId(), "事业部id不能为空");
        Department department = departmentService.getById(dto.getDepartmentId());
        Assert.notNull(department, "未知事业部");
        String priceBillingStandardIdStr = department.getPriceBillingStandardId();
        Assert.notBlank(priceBillingStandardIdStr, "事业部未绑定加工计费标准");

        // 查询事业部绑定的品牌
        List<String> brandList = Arrays.asList(department.getWlnBrand().split(","));
        if (ObjectUtil.isEmpty(brandList)) {
            return Collections.emptyList();
        }

        IWrapper<SkuSpec> wrapper = getWrapper();
        wrapper.orderByDesc("ss", SkuSpec::getId);
        wrapper.in("s", Sku::getBrand, brandList);
        wrapper.like("ss", SkuSpec::getCode, dto.getSkuSpecCode());
        wrapper.like("bs", BomSpec::getCode, dto.getBomSpecCode());
        wrapper.like("bs", BomSpec::getColour, dto.getColour());
        wrapper.eq("b", Bom::getChromatophore, dto.getChromatophore());
        wrapper.isNotNull("ss.bom_spec_id");
        List<SkuSpecQuotationVo> skuSpecList = this.baseMapper.getSkuSpecQuotationList(wrapper);

        if (skuSpecList.isEmpty()) {
            return Collections.emptyList();
        }

        // 查询加工计费标准数据
        List<String> priceBillingStandardIdList = Arrays.stream(priceBillingStandardIdStr.split(",")).collect(Collectors.toList());
        List<PriceBillingStandard> priceBillingStandardList = priceBillingStandardService.listByIds(priceBillingStandardIdList);
        List<Long> priceBillingStandardIds = priceBillingStandardList.stream().map(BaseIdPo::getId).collect(Collectors.toList());

        Map<Long, List<PriceBillingStandardDetail>> priceDetailMap = priceBillingStandardDetailService.mapKGroup(
                PriceBillingStandardDetail::getPriceBillingStandardId,
                q -> q.in(PriceBillingStandardDetail::getPriceBillingStandardId, priceBillingStandardIds));

        // 获取sku关联的所有bom数据
        List<Long> skuSpecIds = skuSpecList.stream().map(SkuSpecQuotationVo::getId).collect(Collectors.toList());

        List<SkuSpecLink> skuSpecLinkList = skuSpecLinkService.list(q -> q.in(SkuSpecLink::getSkuSpecId, skuSpecIds));
        Map<Long, Map<Integer, List<SkuSpecLink>>> skuSpecLinkBomSpecMap = skuSpecLinkList.stream().collect(
                Collectors.groupingBy(SkuSpecLink::getSkuSpecId, Collectors.groupingBy(SkuSpecLink::getType)));

        Set<Long> bomSpecIds = skuSpecList.stream().map(SkuSpecQuotationVo::getBomSpecId).collect(Collectors.toSet());
        bomSpecIds.addAll(skuSpecLinkList.stream().map(SkuSpecLink::getBomSpecId).collect(Collectors.toList()));
        Map<Long, BomSpecBo> bomSpecBoMap = this.getBomSpecBoByIdList(bomSpecIds);

        Map<String, String> bomChromatophoreMap = dictCommonDataService.list(q -> q.eq(DictCommonData::getDictCode, "bom_chromatophore"))
                .stream().collect(Collectors.toMap(DictCommonData::getDictKey, DictCommonData::getDictValue));


        for (SkuSpecQuotationVo vo : skuSpecList) {
            BomSpecBo bomSpecBo = bomSpecBoMap.get(vo.getBomSpecId());

            // 赋值
            vo.setChromatophore(bomChromatophoreMap.get(vo.getChromatophore()));
            vo.setBomSpecUnitPrice(bomSpecBo.getInternalSellingPrice());
            vo.setPackagingMaterialCost(BigDecimal.ZERO);
            vo.setLineProcessingFee(BigDecimal.ZERO);
            vo.setLogoProcessingFee(BigDecimal.ZERO);
            vo.setIssueFee(BigDecimal.ZERO);
            vo.setDeliveryMaterialsFee(BigDecimal.ZERO);
            vo.setPackingLabor(BigDecimal.ZERO);
            vo.setManagementFee(BigDecimal.ZERO);

            // 获取包材费
            Map<Integer, List<SkuSpecLink>> linkMap = skuSpecLinkBomSpecMap.getOrDefault(vo.getId(), Collections.emptyMap());
            List<SkuSpecLink> packagingLinkList = linkMap.getOrDefault(1, Collections.emptyList());
            if (ObjectUtil.isNotEmpty(packagingLinkList)) {
                BigDecimal packagingMaterialCost = packagingLinkList.stream().map(item -> {
                    BomSpecBo skuSpecLinkBomSpec = bomSpecBoMap.get(item.getBomSpecId());
                    if (skuSpecLinkBomSpec == null) {
                        return BigDecimal.ZERO;
                    }
                    return skuSpecLinkBomSpec.getInternalSellingPrice().multiply(item.getQuantity());
                }).reduce(BigDecimal.ZERO, BigDecimal::add);
                vo.setPackagingMaterialCost(packagingMaterialCost);
            }

            // 获取加工计费标准
            List<PriceBillingStandardDetail> priceBillingStandardDetailList = new ArrayList<>();
            if (StrUtil.isNotBlank(bomSpecBo.getBomSpecies())) {
                priceBillingStandardList.stream()
                        .filter(item -> Objects.equals(item.getSpecies(), bomSpecBo.getBomSpecies()))
                        .findAny()
                        .ifPresent(item -> priceBillingStandardDetailList.addAll(priceDetailMap.getOrDefault(item.getId(), Collections.emptyList())));
            }

            for (PriceBillingStandardDetail priceBillingStandardDetail : priceBillingStandardDetailList) {
                String chargeItem = priceBillingStandardDetail.getChargeItem();
                BigDecimal chargePrice = ObjectUtil.defaultIfNull(priceBillingStandardDetail.getChargePrice(), BigDecimal.ZERO);
                switch (chargeItem) {
                    case "10":
                        vo.setLineProcessingFee(chargePrice);
                        break;
                    case "20":
                        vo.setLogoProcessingFee(chargePrice);
                        break;
                    case "40":
                        vo.setPackingLabor(chargePrice);
                        break;
                    case "50":
                        vo.setIssueFee(chargePrice);
                        break;
                    case "70":
                        vo.setManagementFee(chargePrice);
                        break;
                }
            }
            // 计算快递包材费
            List<SkuSpecLink> deliveryLinkList = linkMap.getOrDefault(2, Collections.emptyList());
            if (deliveryLinkList.isEmpty()) {
                vo.setDeliveryMaterialsFee(BigDecimal.ZERO);
            } else {
                BigDecimal deliveryMaterialsFee = deliveryLinkList.stream()
                        .map(item -> {
                            BomSpecBo bomSpec = bomSpecBoMap.get(item.getBomSpecId());
                            if (bomSpec == null) {
                                return BigDecimal.ZERO;
                            }
                            return item.getQuantity().multiply(bomSpec.getInternalSellingPrice());
                        })
                        .reduce(BigDecimal.ZERO, BigDecimal::add);
                vo.setDeliveryMaterialsFee(deliveryMaterialsFee);
            }

            // 计算Sku单价
            vo.setSkuSpecUnitPrice(
                    vo.getBomSpecUnitPrice()
                            .add(vo.getPackagingMaterialCost())
                            .add(vo.getIssueFee())
                            .add(vo.getPackingLabor())
                            .add(vo.getDeliveryMaterialsFee())
                            .add(vo.getManagementFee()));
            vo.setSkuSpecUnitPriceLogo(vo.getSkuSpecUnitPrice().add(vo.getLogoProcessingFee()));
            vo.setSkuSpecUnitPriceLine(vo.getSkuSpecUnitPrice().add(vo.getLineProcessingFee()));
        }

        return skuSpecList;
    }

}
