package com.sd.business.service.statement.impl;

import com.alibaba.excel.metadata.Head;
import com.alibaba.excel.write.merge.AbstractMergeStrategy;
import com.sd.business.entity.statement.vo.DocumentByOrderVo;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.util.CellRangeAddress;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

public class SalesOutWarehouseDetailsExcelCellMergeStrategy extends AbstractMergeStrategy {

    // 最大行数
    private final int maxRow;

    // sku需要合并的列
    private final List<Integer> skuColIndex = Arrays.asList(6, 7, 8, 22, 23);
    // sku无需合并的行
    private final List<Integer> skuRowIndex = new ArrayList<>();

    // 订单需要合并的列
    private final List<Integer> orderColIndex = Arrays.asList(0, 1, 2, 3, 4, 5, 24, 25);
    // 订单无需合并的行
    private final List<Integer> orderRowIndex = new ArrayList<>();

    public SalesOutWarehouseDetailsExcelCellMergeStrategy(List<DocumentByOrderVo> documentByOrderVoList) {

        // 无需合并的行数
        int mergeRowIndex = 1;
        skuRowIndex.add(mergeRowIndex);
        orderRowIndex.add(mergeRowIndex);

        for (DocumentByOrderVo documentByOrderVo : documentByOrderVoList) {
            List<DocumentByOrderVo.SkuSpec> skuSpecList = documentByOrderVo.getSkuSpecList();

            // 赋值订单无需合并的行
            Integer size = skuSpecList.stream().map(item -> item.getBomSpecList().size()).reduce(0, Integer::sum);
            int orderRowIndexNumber = orderRowIndex.isEmpty() ? mergeRowIndex : orderRowIndex.get(orderRowIndex.size() - 1);
            orderRowIndex.add(size + orderRowIndexNumber);

            // 赋值sku无需合并的行
            for (DocumentByOrderVo.SkuSpec skuSpec : skuSpecList) {
                int bomSpecListSize = skuSpec.getBomSpecList().size();
                int skuRowIndexSize = skuRowIndex.size();
                int skuRowIndexNumber = skuRowIndexSize == 0 ? mergeRowIndex : skuRowIndex.get(skuRowIndexSize - 1);
                skuRowIndex.add(bomSpecListSize + skuRowIndexNumber);
            }
        }

        maxRow = skuRowIndex.get(skuRowIndex.size() - 1);
    }

    @Override
    protected void merge(Sheet sheet, Cell cell, Head head, Integer relativeRowIndex) {

        // 当前行列
        int curRowIndex = cell.getRowIndex();

        // 判断是否是最后一行
        if (curRowIndex != maxRow) {
            return;
        }

        int curColIndex = cell.getColumnIndex();

        // 合并订单列
        if (orderColIndex.contains(curColIndex)) {
            for (int i = 1; i < orderRowIndex.size(); i++) {
                int beginRow = orderRowIndex.get(i - 1) + 1;
                Integer endRow = orderRowIndex.get(i);
                mergeWithPrevRow(sheet, beginRow, endRow, curColIndex);
            }
        }

        // 合并sku列
        else if (skuColIndex.contains(curColIndex)) {
            for (int i = 1; i < skuRowIndex.size(); i++) {
                int beginRow = skuRowIndex.get(i - 1) + 1;
                Integer endRow = skuRowIndex.get(i);
                mergeWithPrevRow(sheet, beginRow, endRow, curColIndex);
            }
        }

    }

    /**
     * 合并单元格
     */
    private void mergeWithPrevRow(Sheet sheet, int beginRow, int endRow, int col) {
        if (beginRow == endRow) {
            return;
        }
        CellRangeAddress cellRangeAddress = new CellRangeAddress(beginRow, endRow, col, col);
        sheet.addMergedRegionUnsafe(cellRangeAddress);
    }

}