package com.sd.business.service.statement.impl;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.toolkit.StringPool;
import com.sd.business.entity.order.enums.OrderClassifyEnum;
import com.sd.business.entity.statement.vo.DocumentByBomVo;
import com.sd.business.entity.statement.vo.DocumentBySkuVo;
import com.sd.business.service.statement.StatementOfAccountExportService;
import com.sd.business.service.statement.StatementOfAccountService;
import com.sd.framework.util.StreamUtil;
import com.sd.framework.util.TemplateExcelUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletResponse;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
public class StatementOfAccountExportServiceImpl implements StatementOfAccountExportService {

    @Autowired
    private HttpServletResponse response;

    @Autowired
    private StatementOfAccountService statementOfAccountService;

    @Override
    public void exportDocumentBySku(List<Long> idList, String departmentName, String beginDate, String endDate, Integer orderClassify) {

        List<DocumentBySkuVo> list = statementOfAccountService.getSkuDocument(idList, orderClassify);

        Map<String, Object> map = new HashMap<>();
        map.put("department", StrUtil.isBlank(departmentName) ? StringPool.EMPTY : departmentName + StringPool.DASH);
        map.put("beginDate", beginDate);
        map.put("endDate", endDate);
        map.put("totalQuantity", StreamUtil.bigDecimalAdd(list, DocumentBySkuVo::getQuantity));
        map.put("totalSubtotal", StreamUtil.bigDecimalAdd(list, DocumentBySkuVo::getSubtotal));

        TemplateExcelUtil.writeBrowser("skuDocument.xlsx", OrderClassifyEnum.getEnum(orderClassify).getValue() + "-sku对账单", response, list, map);
    }

    @Override
    public void exportDocumentByBom(List<Long> idList, String departmentName, String beginDate, String endDate, Integer orderClassify) {

        List<DocumentByBomVo> list = statementOfAccountService.getBomDocument(idList, orderClassify);

        Map<String, Object> map = new HashMap<>();
        map.put("department", StrUtil.isBlank(departmentName) ? StringPool.EMPTY : departmentName + StringPool.DASH);
        map.put("beginDate", beginDate);
        map.put("endDate", endDate);
        map.put("totalSubtotal", StreamUtil.bigDecimalAdd(list, DocumentByBomVo::getSubtotal));
        map.put("totalLaserLogoSummary", StreamUtil.bigDecimalAdd(list, DocumentByBomVo::getLaserLogoSummary));
        map.put("totalLaserMitochondrialSummary", StreamUtil.bigDecimalAdd(list, DocumentByBomVo::getLaserMitochondrialSummary));
        map.put("totalLssueFeeSummary", StreamUtil.bigDecimalAdd(list, DocumentByBomVo::getLssueFeeSummary));
        map.put("totalDeliveryMaterialsFeeSummary", StreamUtil.bigDecimalAdd(list, DocumentByBomVo::getDeliveryMaterialsFeeSummary));
        map.put("totalPackingLaborSummary", StreamUtil.bigDecimalAdd(list, DocumentByBomVo::getPackingLaborSummary));
        map.put("totalManagementFeeSummary", StreamUtil.bigDecimalAdd(list, DocumentByBomVo::getManagementFeeSummary));

        TemplateExcelUtil.writeBrowser("bomDocument.xlsx", OrderClassifyEnum.getEnum(orderClassify).getValue() + "-bom对账单", response, list, map);
    }

}
