package com.sd.business.strategy.impl;

import cn.hutool.core.io.IoUtil;
import com.alibaba.excel.EasyExcel;
import com.sd.business.strategy.ExcelExportStrategy;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.util.List;
import java.util.function.Supplier;

public class DefaultExportStrategy<T> implements ExcelExportStrategy<List<T>> {

    private final Class<T> aClass;

    private final Supplier<List<T>> supplier;

    private final String sheetName;

    public DefaultExportStrategy(Supplier<List<T>> supplier, Class<T> aClass, String sheetName) {
        this.supplier = supplier;
        this.aClass = aClass;
        this.sheetName = sheetName;
    }

    @Override
    public List<T> getData() {
        return supplier.get();
    }

    @Override
    public InputStream getInputStream(List<T> data) {

        ByteArrayOutputStream os = null;

        try {
            os = new ByteArrayOutputStream();
            EasyExcel.write(os, aClass).sheet(sheetName).doWrite(data);
            return new ByteArrayInputStream(os.toByteArray());
        } finally {
            IoUtil.close(os);
        }

    }
}
