import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.io.FileUtil;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.ExcelWriter;
import com.alibaba.excel.metadata.Head;
import com.alibaba.excel.write.merge.AbstractMergeStrategy;
import com.alibaba.excel.write.metadata.WriteSheet;
import com.alibaba.excel.write.metadata.fill.FillConfig;
import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.dynamic.datasource.annotation.DSTransactional;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.ruoyi.common.exception.ServiceException;
import com.sd.SdApplication;
import com.sd.business.entity.order.po.OrderInfo;
import com.sd.business.entity.production.dto.StockPreparationDto;
import com.sd.business.entity.production.vo.UncompletedVo;
import com.sd.business.entity.statement.bo.ExportDocumentByOrderBo;
import com.sd.business.entity.statement.dto.GetDocumentDto;
import com.sd.business.entity.statement.vo.DocumentByOrderVo;
import com.sd.business.service.order.OrderService;
import com.sd.business.service.production.StockPreparationService;
import com.sd.business.service.statement.StatementOfAccountMergeService;
import com.sd.wln.service.WlnStatementOfAccount;
import com.sd.wln.util.WlnUtil;
import lombok.SneakyThrows;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.util.CellRangeAddress;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.core.io.ClassPathResource;
import org.springframework.test.context.junit4.SpringRunner;

import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.math.BigDecimal;
import java.nio.charset.Charset;
import java.util.*;
import java.util.stream.Collectors;

@RunWith(SpringRunner.class)
@SpringBootTest(classes = SdApplication.class, webEnvironment = SpringBootTest.WebEnvironment.DEFINED_PORT)
public class TestList {

    @Autowired
    private StockPreparationService stockPreparationService;

    @Autowired
    private WlnStatementOfAccount wlnStatementOfAccount;

    @Autowired
    private OrderService orderService;

    @DSTransactional
    @Test
    public void test2() {
        wlnStatementOfAccount.createStatementOfAccount();
    }

    @Test
    public void test() {
        StockPreparationDto stockPreparationDto = new StockPreparationDto();
        stockPreparationDto.setDepartmentId(1689164627162529793L);
        List<UncompletedVo> uncompletedVos = stockPreparationService.uncompletedList(stockPreparationDto);
        System.out.println();


        String sql = "update in_out_storage_details i INNER JOIN in_out_storage_bom iosb on i.bom_spec_id = iosb.bom_spec_id\n" +
                "set i.quantity = i.quantity + iosb.quantity";


        String sql2 = "SELECT DISTINCT\n" +
                "\twln_code \n" +
                "FROM\n" +
                "\torder_info oi\n" +
                "\tLEFT JOIN order_sku os ON oi.id = os.order_id \n" +
                "WHERE\n" +
                "\toi.department_id = 1689164627162529793\n" +
                "\tand os.blueprint is null\n" +
                "\tand os.production_document is null";

        String sql3 = "UPDATE order_sku os\n" +
                "INNER JOIN sku_spec ss ON os.sku_spec_id = ss.id \n" +
                "SET os.blueprint = ss.design_img_url,\n" +
                "os.production_document = ss.shared_folder";

    }

    @Test
    public void a() {
        String warehouseCode = "B014";

        long endTime = new Date().getTime();
        long startTime;

        OrderInfo orderInfo = orderService.getOne(q -> q
                .eq(OrderInfo::getWlnStorageCode, warehouseCode)
                .orderByDesc(OrderInfo::getWlnModifyTimestamp));

        List<JSONObject> list = new ArrayList<>();
        startTime = endTime - 1000 * 60 * 60 * 24 * 7;
        getOrderList(startTime, endTime, warehouseCode, list);

        System.out.println(list);
    }

    private void getOrderList(Long startTime, Long endTime, String warehouseCode, List<JSONObject> list) {
        int page = 1;
        int size;
        do {
            try {
                List<JSONObject> itemList = WlnUtil.getOrderList(page, 200, startTime, endTime, warehouseCode);
                page++;
                size = itemList.size();
                list.addAll(itemList);
            } catch (Exception e) {
                throw new ServiceException("订单同步失败");
            }
        } while (size >= 200);
    }


    @Autowired
    private StatementOfAccountMergeService statementOfAccountMergeService;

    @Test
    public void b() {
        String a = "{\n" +
                "    \"idGroupConcat\": \"1699739286011727873,1702638389020454914,1700101673936044034,1700464061822562305,1702276001603575809,1700826449696530434,1701928713001660417,1701551225520091137,1701106726288019458,1701106725822451713,1701106725755342850,1701106725625319426,1701106725558210565,1701106725558210564,1699376899223990273\"\n" +
                "}";
        GetDocumentDto dto = JSON.parseObject(a).toJavaObject(GetDocumentDto.class);


        List<DocumentByOrderVo> documentByOrder = statementOfAccountMergeService.getDocumentByOrder(dto);

        List<DocumentByOrderVo> documentByOrderVoStream = documentByOrder.stream().filter(item -> {
            List<DocumentByOrderVo.SkuSpec> skuSpecList = item.getSkuSpecList();
            return skuSpecList.stream().anyMatch(i -> i.getSkuSpecCode().equals("FY088-3-08AL39B"));
        }).collect(Collectors.toList());


        Map<String, BigDecimal> collect = documentByOrderVoStream.stream()
                .collect(Collectors.toMap(
                        DocumentByOrderVo::getWlnCode,
                        item -> item.getSkuSpecList().stream()
                                .filter(i -> i.getSkuSpecCode().equals("FY088-3-08AL39B"))
                                .map(DocumentByOrderVo.SkuSpec::getQuantity).reduce(BigDecimal.ZERO, BigDecimal::add))
                );


        System.out.println(JSON.toJSONString(collect));
    }

    // public static void main(String[] args) {
    //     String a = "{\n" +
    //             "    \"idGroupConcat\": \"1699739286011727873,1702638389020454914,1700101673936044034,1700464061822562305,1702276001603575809,1700826449696530434,1701928713001660417,1701551225520091137,1701106726288019458,1701106725822451713,1701106725755342850,1701106725625319426,1701106725558210565,1701106725558210564,1699376899223990273\"\n" +
    //             "}";
    //     System.out.println(a);
    // }


    @SneakyThrows
    public static void main(String[] args) {

        String str2 = FileUtil.readString("E:\\新建文本文档2.txt", Charset.defaultCharset());
        List<DocumentByOrderVo> list = JSON.parseArray(str2).toJavaList(DocumentByOrderVo.class);

        String str = FileUtil.readString("E:\\新建文本文档.txt", Charset.defaultCharset());

        JSONObject json = JSON.parseObject(str);
        List<ExportDocumentByOrderBo> exportDocumentByOrderBos = (List<ExportDocumentByOrderBo>) json.remove("exportDocumentByOrderBos");

        String filePath = "tempExcel" + File.separator + IdWorker.getId() + ".xlsx";
        File tempExcel = new File("tempExcel");
        if (!tempExcel.exists()) {
            tempExcel.mkdir();
        }
        FileOutputStream os = new FileOutputStream(filePath);

        ClassPathResource classPathResource = new ClassPathResource("template" + File.separator + "orderDocument.xlsx");
        InputStream is = classPathResource.getInputStream();
        ExcelWriter excelWriter = EasyExcel.write(os).withTemplate(is).build();
        try {
            A strategy = new A(list);
            WriteSheet writeSheet = EasyExcel.writerSheet().registerWriteHandler(strategy).build();
            FillConfig fillConfig = FillConfig.builder().forceNewRow(Boolean.TRUE).build();

            System.out.println("写入1");
            excelWriter.fill(json, fillConfig, writeSheet);
            System.out.println("写入2");
            List<List<ExportDocumentByOrderBo>> ssss = ListUtil.partition(exportDocumentByOrderBos, 10);
            for (List<ExportDocumentByOrderBo> documentByOrderBos : ssss) {
                excelWriter.fill(documentByOrderBos, fillConfig, writeSheet);
            }

            System.out.println("写入完成");
        } finally {
            if (excelWriter != null) {
                excelWriter.finish();
                excelWriter.close();
            }
            os.close();
            is.close();
        }

    }

    private static class A extends AbstractMergeStrategy {

        // 最大行数
        private final int maxRow;

        // sku需要合并的列
        private final List<Integer> skuColIndex = Arrays.asList(3, 4, 5, 16, 17);
        // sku无需合并的行
        private final List<Integer> skuRowIndex = new ArrayList<>();

        // 订单需要合并的列
        private final List<Integer> orderColIndex = Arrays.asList(0, 1, 2, 18, 19);
        // 订单无需合并的行
        private final List<Integer> orderRowIndex = new ArrayList<>();


        public A(List<DocumentByOrderVo> documentByOrderVoList) {

            // 无需合并的行数
            int mergeRowIndex = 2;
            skuRowIndex.add(mergeRowIndex);
            orderRowIndex.add(mergeRowIndex);

            for (DocumentByOrderVo documentByOrderVo : documentByOrderVoList) {
                List<DocumentByOrderVo.SkuSpec> skuSpecList = documentByOrderVo.getSkuSpecList();

                // 赋值订单无需合并的行
                Integer size = skuSpecList.stream().map(item -> item.getBomSpecList().size()).reduce(0, Integer::sum);
                int orderRowIndexNumber = orderRowIndex.size() == 0 ? mergeRowIndex : orderRowIndex.get(orderRowIndex.size() - 1);
                orderRowIndex.add(size + orderRowIndexNumber);

                // 赋值sku无需合并的行
                for (DocumentByOrderVo.SkuSpec skuSpec : skuSpecList) {
                    int bomSpecListSize = skuSpec.getBomSpecList().size();
                    int skuRowIndexSize = skuRowIndex.size();
                    int skuRowIndexNumber = skuRowIndexSize == 0 ? mergeRowIndex : skuRowIndex.get(skuRowIndexSize - 1);
                    skuRowIndex.add(bomSpecListSize + skuRowIndexNumber);
                }
            }

            maxRow = skuRowIndex.get(skuRowIndex.size() - 1);

            System.out.println();
        }

        @Override
        protected void merge(Sheet sheet, Cell cell, Head head, Integer relativeRowIndex) {

            // 当前行列
            int curRowIndex = cell.getRowIndex();

            if (curRowIndex != maxRow) {
                return;
            }

            int curColIndex = cell.getColumnIndex();

            // 合并订单列
            if (orderColIndex.contains(curColIndex)) {
                for (int i = 1; i < orderRowIndex.size(); i++) {
                    Integer beginRow = orderRowIndex.get(i - 1);
                    Integer endRow = orderRowIndex.get(i);
                    mergeWithPrevRow(sheet, beginRow, endRow, curColIndex);
                }
            }

            // 合并sku列
            else if (skuColIndex.contains(curColIndex)) {
                for (int i = 1; i < skuRowIndex.size(); i++) {
                    Integer beginRow = skuRowIndex.get(i - 1);
                    Integer endRow = skuRowIndex.get(i);
                    mergeWithPrevRow(sheet, beginRow, endRow, curColIndex);
                }
            }

        }

        /**
         * 合并单元格
         */
        private void mergeWithPrevRow(Sheet sheet, int beginRow, int endRow, int col) {
            CellRangeAddress cellRangeAddress = new CellRangeAddress(beginRow + 1, endRow, col, col);
            sheet.addMergedRegionUnsafe(cellRangeAddress);
        }

    }


}
